/*------------------------------------------------------blackboard.c----------*/
/******************************************************************************/
/*                                                                            */
/*                                                                            */
/*                              Vartalaap                                     */
/*----------------------------------------------------------------------------*/
/*                  Multimedia Multiparty  Conferencing System                */
/*                                                                            */
/*                                                                            */
/*                  Written by:VIJAY TALATI                                   */
/*                             ERNet Lab,                                     */
/*                             Department of Computer Science,                */
/*                             Indian Institute of Technology, Bombay 400076. */
/*                             E-mail : vijay@gateway.iitb.ernet.in           */
/*                                                                            */
/******************************************************************************/

#include "vartalaap.h"
#include "posdefs.h"
#include "bb.h"
#include "../common/talklib.h"
#include "../common/protodefs.h"


void	process_circ () ;
void	process_rect () ;
void	process_arc () ;
void	process_grid () ;
void	process_polyline () ;
void	process_polygon () ;
void	process_erase () ;
void	process_text () ;
void	process_todo () ;
void	process_filesave () ;
void	bb_event_proc () ;
void	bb_quit () ;
void	draw_bb() ;
void	draw_grid() ;
void	fill_points() ;
void	calculate_obj_size() ;
void	file_menu_proc() ;
void	color_proc() ;
void	fill_style_proc() ;
void	line_style_proc() ;
void	line_width_proc() ;
void	font_family_proc() ;
void	font_style_proc() ;
void	font_size_proc() ;

void	setfontfamily() ;
void	setfontstyle() ;
void	setfontscale() ;

#include	"buttons.h"
#include	"fills.h"

 
static char fill_images[NUMBER_OF_PATTERNS][8] = {
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
    {0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00},
    {0x00, 0x40, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00},
    {0x00, 0x40, 0x00, 0x10, 0x00, 0x04, 0x00, 0x00},
    {0x00, 0x44, 0x00, 0x00, 0x00, 0x44, 0x00, 0x00},
    {0x00, 0x44, 0x00, 0x10, 0x00, 0x44, 0x00, 0x00},
    {0x88, 0x00, 0x22, 0x00, 0x88, 0x00, 0x22, 0x00},
    {0x88, 0x00, 0x32, 0x00, 0x88, 0x00, 0x23, 0x00},
    {0xc4, 0x00, 0x31, 0x00, 0x8c, 0x00, 0x23, 0x00},
    {0x99, 0x00, 0xcc, 0x00, 0x66, 0x00, 0x33, 0x00},
    {0x55, 0x00, 0x55, 0x00, 0x55, 0x00, 0x55, 0x00},
    {0x55, 0x22, 0x55, 0x08, 0x55, 0x22, 0x55, 0x00},
    {0x55, 0xa2, 0x55, 0x88, 0x55, 0xa2, 0x55, 0x2a},
    {0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa},
    {0x55, 0xaa, 0x77, 0xaa, 0x55, 0xaa, 0xdd, 0xaa},
    {0x55, 0xae, 0x5f, 0xae, 0x55, 0xea, 0xf5, 0xea},
    {0x5d, 0xb6, 0x7f, 0xd5, 0x7f, 0xb6, 0x5d, 0xaa},
    {0x77, 0xff, 0x77, 0xaa, 0x77, 0xff, 0x77, 0xaa},
    {0x77, 0xdd, 0x7f, 0xbe, 0x7f, 0xdd, 0x77, 0xaa},
    {0x77, 0xff, 0x7f, 0xbe, 0x7f, 0xff, 0x77, 0xaa},
    {0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff},

	} ;

#define	NUMBER_OF_FONT_FAMILIES	11
static	char	*font_families []= {
 					FONT_FAMILY_DEFAULT, FONT_FAMILY_DEFAULT_FIXEDWIDTH,
            		FONT_FAMILY_LUCIDA, FONT_FAMILY_LUCIDA_FIXEDWIDTH,
            		FONT_FAMILY_ROMAN, FONT_FAMILY_SERIF, FONT_FAMILY_COUR,
            		FONT_FAMILY_CMR, FONT_FAMILY_GALLENT,
            		FONT_FAMILY_OLGLYPH, FONT_FAMILY_OLCURSOR
					} ;

#define	NUMBER_OF_FONT_STYLES	5
static	char	*font_styles []= {
            		FONT_STYLE_DEFAULT, FONT_STYLE_NORMAL, FONT_STYLE_BOLD,
            		FONT_STYLE_ITALIC, FONT_STYLE_BOLD_ITALIC
					} ;

static	char	*font_sizes []= {
            		"Small", "Medium", "Large", "X-Large"
					} ;

extern	char	*color_names[] ;
extern	unsigned long	col_pixel[] ;

/*------------------------------------------------make_bboard()---------------*/
Frame	make_bboard( grp_id)
int	grp_id ;
{
	Window		xwin ;
	char		dash_list[2] ;
	Menu		fillstylemenu ;
	Menu		linestylemenu ;
	Menu		linewidthmenu ;
	Menu		fontfamilymenu ;
	Menu		fontstylemenu ;
	Menu		fontsizemenu ;
	Menu		colormenu ;
	Menu		filemenu ;
	Xv_font		font ;
	XGCValues	gcvalues ;
	Server_image	circle_svr_image,
					polygon_svr_image,
					polyline_svr_image,
					text_svr_image,
					arc_svr_image ,
					spline_svr_image,
					polyspline_svr_image,
					rectangle_svr_image;
	int			i ;
	int			*mgrp_id_ptr ;

	/*Creating the server images */	
	for (i=0; i < NUMBER_OF_PATTERNS; i++)
	{
		fimg[i] = (Server_image)xv_create(XV_NULL, SERVER_IMAGE,
				XV_WIDTH,	16,
				XV_HEIGHT,	16,
				SERVER_IMAGE_BITS,	fill_bits[i],
				NULL) ;
	}
	arc_svr_image = (Server_image)xv_create(XV_NULL, SERVER_IMAGE,
				XV_WIDTH,	16,
				XV_HEIGHT,	16,
				SERVER_IMAGE_BITS,	arc_bits,
				NULL) ;
	circle_svr_image = (Server_image)xv_create(XV_NULL, SERVER_IMAGE,
				XV_WIDTH,	16,
				XV_HEIGHT,	16,
				SERVER_IMAGE_BITS,	circle_bits,
				NULL) ;
	rectangle_svr_image = (Server_image)xv_create(XV_NULL, SERVER_IMAGE,
				XV_WIDTH,	16,
				XV_HEIGHT,	16,
				SERVER_IMAGE_BITS,	rectangle_bits,
				NULL) ;
	polyline_svr_image = (Server_image)xv_create(XV_NULL, SERVER_IMAGE,
				XV_WIDTH,	16,
				XV_HEIGHT,	16,
				SERVER_IMAGE_BITS,	polyline_bits,
				NULL) ;
	polygon_svr_image = (Server_image)xv_create(XV_NULL, SERVER_IMAGE,
				XV_WIDTH,	16,
				XV_HEIGHT,	16,
				SERVER_IMAGE_BITS,	polygon_bits,
				NULL) ;
	spline_svr_image = (Server_image)xv_create(XV_NULL, SERVER_IMAGE,
				XV_WIDTH,	16,
				XV_HEIGHT,	16,
				SERVER_IMAGE_BITS,	spline_bits,
				NULL) ;
	polyspline_svr_image = (Server_image)xv_create(XV_NULL, SERVER_IMAGE,
				XV_WIDTH,	16,
				XV_HEIGHT,	16,
				SERVER_IMAGE_BITS,	polyspline_bits,
				NULL) ;
	text_svr_image = (Server_image)xv_create(XV_NULL, SERVER_IMAGE,
				XV_WIDTH,	16,
				XV_HEIGHT,	16,
				SERVER_IMAGE_BITS,	text_bits,
				NULL) ;
 
	BBoard[grp_id].bb_frame = (Frame) xv_create( 
		grp_box_ptr[grp_id], FRAME,
		XV_HEIGHT,	BBFRM_HEIGHT, 
		XV_WIDTH,	BBFRM_WIDTH, 
		XV_X,		BBFRM_X ,
		XV_Y,		BBFRM_Y ,
		FRAME_LABEL,	"BlackBoard",
		FRAME_DONE_PROC, bb_quit,
		NULL) ;

	BBoard[grp_id].bb_canvas = (Canvas) xv_create( 
		BBoard[grp_id].bb_frame, CANVAS,
		CANVAS_X_PAINT_WINDOW,	TRUE,
		CANVAS_REPAINT_PROC,	draw_bb,
		/*CANVAS_RETAINED,	FALSE ,*/
		CANVAS_WIDTH,	BBFRM_WIDTH,
		CANVAS_HEIGHT,	BBFRM_HEIGHT,
		XV_HEIGHT,	BBFRM_HEIGHT,
		XV_WIDTH,	BBFRM_WIDTH , 
		XV_X,		0,
		XV_Y,      2*BTN_PNL_HEIGHT+30,
		CANVAS_AUTO_SHRINK, FALSE,
		CANVAS_AUTO_EXPAND, FALSE,
		/*OPENWIN_AUTO_CLEAR,	FALSE,*/
		NULL ) ;
 
	/* the panel to contain buttons */

	BBoard[grp_id].bb_panel = (Panel)xv_create( 
		BBoard[grp_id].bb_frame, PANEL,
		XV_HEIGHT,      2*BTN_PNL_HEIGHT+30,
		XV_WIDTH,		BBFRM_WIDTH,
		XV_X,	0,
		XV_Y,	0,
		NULL) ;


	(void)xv_create( BBoard[grp_id].bb_panel, PANEL_BUTTON,
				PANEL_LABEL_IMAGE,		arc_svr_image,
                PANEL_NOTIFY_PROC,      process_todo,
                NULL) ;
	(void)xv_create( BBoard[grp_id].bb_panel, PANEL_BUTTON,
                PANEL_LABEL_IMAGE,     circle_svr_image,
                PANEL_NOTIFY_PROC,      process_circ,
                NULL) ;
	(void)xv_create( BBoard[grp_id].bb_panel, PANEL_BUTTON,
                PANEL_LABEL_IMAGE,     	polygon_svr_image ,
                PANEL_NOTIFY_PROC,      process_polygon,
                NULL) ;
	(void)xv_create( BBoard[grp_id].bb_panel, PANEL_BUTTON,
                PANEL_LABEL_IMAGE,	 	polyline_svr_image ,
                PANEL_NOTIFY_PROC,      process_polyline,
                NULL) ;
	(void)xv_create( BBoard[grp_id].bb_panel, PANEL_BUTTON,
                PANEL_LABEL_IMAGE,     rectangle_svr_image,
                PANEL_NOTIFY_PROC,      process_rect,
                NULL) ;
	(void)xv_create( BBoard[grp_id].bb_panel, PANEL_BUTTON,
                PANEL_LABEL_IMAGE,     spline_svr_image,
                PANEL_NOTIFY_PROC,      process_todo,
                NULL) ;
	(void)xv_create( BBoard[grp_id].bb_panel, PANEL_BUTTON,
                PANEL_LABEL_IMAGE,     polyspline_svr_image,
                PANEL_NOTIFY_PROC,      process_todo,
                NULL) ;
	(void)xv_create( BBoard[grp_id].bb_panel, PANEL_BUTTON,
                PANEL_LABEL_IMAGE,     text_svr_image,
                PANEL_NOTIFY_PROC,      process_text,
                NULL) ;
	(void)xv_create( BBoard[grp_id].bb_panel, PANEL_BUTTON,
                PANEL_LABEL_STRING,     "Erase",
                PANEL_NOTIFY_PROC,      process_erase,
                NULL) ;

	filemenu = (Menu)xv_create(NULL, MENU,
				MENU_NOTIFY_PROC,	file_menu_proc,
				MENU_CLIENT_DATA,	BBoard[grp_id].bb_panel ,
				MENU_STRINGS, "Write","Read", NULL,
				NULL) ;
	(void)xv_create( BBoard[grp_id].bb_panel, PANEL_BUTTON,
                PANEL_LABEL_STRING,     "File",
                PANEL_ITEM_MENU,     filemenu,
                PANEL_NOTIFY_PROC,      process_filesave,
                NULL) ;

/******
	(void)xv_create( BBoard[grp_id].bb_panel, PANEL_BUTTON,
                PANEL_LABEL_STRING,     "Grid",
				XV_X,			0 ,
				XV_Y,			40,
                PANEL_NOTIFY_PROC,      process_grid,
                NULL) ;
****/

	colormenu = (Menu)xv_create(NULL, MENU_CHOICE_MENU,
				MENU_NOTIFY_PROC,	color_proc ,
				MENU_CLIENT_DATA,	BBoard[grp_id].bb_panel ,
				MENU_STRINGS,	"Black", "Blue", "Brown","Cyan","Green", 
					"Khaki","Orange", "Red", "Violet", "Yellow", 
					NULL,
				NULL) ;
	(void)xv_create( BBoard[grp_id].bb_panel, PANEL_BUTTON,
                PANEL_LABEL_STRING,     "Color",
				XV_X,			0 ,
				XV_Y,			40,
                PANEL_ITEM_MENU,     	colormenu ,
                NULL) ;

	fillstylemenu = (Menu)xv_create(NULL, MENU_CHOICE_MENU,
				MENU_NOTIFY_PROC,	fill_style_proc ,	
				MENU_CLIENT_DATA,	BBoard[grp_id].bb_panel ,
				MENU_NCOLS,		4,
				MENU_NROWS,		6,
				MENU_IMAGES,	fimg[0],fimg[1],fimg[2],
								fimg[3], fimg[4], fimg[5],  
								fimg[6], fimg[7], fimg[8], 
								fimg[9], fimg[10], fimg[11],
								fimg[12], fimg[13], fimg[14],
								fimg[15], fimg[16], fimg[17], 
								fimg[18], fimg[19], fimg[20],NULL,
				/*MENU_GEN_PIN_WINDOW, BBoard[grp_id].bb_frame, "Patterns ",*/
				NULL) ;

	(void)xv_create( BBoard[grp_id].bb_panel, PANEL_BUTTON,
                PANEL_LABEL_STRING,     "FillStyle",
				XV_X,			68 ,
				XV_Y,			40,
                PANEL_ITEM_MENU,     	fillstylemenu ,
                NULL) ;

	linestylemenu = (Menu)xv_create(NULL, MENU_CHOICE_MENU,
				MENU_NOTIFY_PROC,	line_style_proc ,
				MENU_CLIENT_DATA,	BBoard[grp_id].bb_panel ,
				MENU_STRINGS,	"Solid", "OnOffDash", "LineDoubleDash", NULL,
				NULL) ;
	(void)xv_create( BBoard[grp_id].bb_panel, PANEL_BUTTON,
                PANEL_LABEL_STRING,     "LineStyle",
				XV_X,			156,
				XV_Y,			40,
                PANEL_ITEM_MENU,     	linestylemenu ,
                NULL) ;

	linewidthmenu = (Menu)xv_create(NULL, MENU_CHOICE_MENU,
				MENU_NOTIFY_PROC,	line_width_proc ,
				MENU_CLIENT_DATA,	BBoard[grp_id].bb_panel ,
				MENU_STRINGS, "1", "2", "3", "4","5", NULL,
				NULL) ;
	(void)xv_create( BBoard[grp_id].bb_panel, PANEL_BUTTON,
                PANEL_LABEL_STRING,     "LineWidth",
				XV_X,			250,
				XV_Y,			40,
                PANEL_ITEM_MENU,     	linewidthmenu ,
                NULL) ;

	fontfamilymenu = (Menu)xv_create(NULL, MENU_CHOICE_MENU,
				MENU_NOTIFY_PROC,	font_family_proc ,
				MENU_CLIENT_DATA,	BBoard[grp_id].bb_panel ,
				MENU_STRINGS, 
 					FONT_FAMILY_DEFAULT, FONT_FAMILY_DEFAULT_FIXEDWIDTH,
            		FONT_FAMILY_LUCIDA, FONT_FAMILY_LUCIDA_FIXEDWIDTH,
            		FONT_FAMILY_ROMAN, FONT_FAMILY_SERIF, FONT_FAMILY_COUR,
            		FONT_FAMILY_CMR, FONT_FAMILY_GALLENT,
            		FONT_FAMILY_OLGLYPH, FONT_FAMILY_OLCURSOR, NULL,
				NULL) ;
	(void)xv_create( BBoard[grp_id].bb_panel, PANEL_BUTTON,
                PANEL_LABEL_STRING,     "FontFamily",
				XV_X,			350,
				XV_Y,			40,
                PANEL_ITEM_MENU,     	fontfamilymenu ,
                NULL) ;

	fontstylemenu = (Menu)xv_create(NULL, MENU_CHOICE_MENU,
				MENU_NOTIFY_PROC,	font_style_proc ,
				MENU_CLIENT_DATA,	BBoard[grp_id].bb_panel ,
				MENU_STRINGS, 
            		FONT_STYLE_DEFAULT, FONT_STYLE_NORMAL, FONT_STYLE_BOLD,
            		FONT_STYLE_ITALIC, FONT_STYLE_BOLD_ITALIC, NULL,
				NULL) ;
	(void)xv_create( BBoard[grp_id].bb_panel, PANEL_BUTTON,
                PANEL_LABEL_STRING,     "FontStyle",
				XV_X,			455,
				XV_Y,			40,
                PANEL_ITEM_MENU,     	fontstylemenu ,
                NULL) ;

	fontsizemenu = (Menu)xv_create(NULL, MENU_CHOICE_MENU,
				MENU_NOTIFY_PROC,	font_size_proc ,
				MENU_CLIENT_DATA,	BBoard[grp_id].bb_panel ,
				MENU_STRINGS, 
            		"Small", "Medium", "Large", "X-Large", NULL,
				NULL) ;
	(void)xv_create( BBoard[grp_id].bb_panel, PANEL_BUTTON,
                PANEL_LABEL_STRING,     "FontSize",
				XV_X,			550,
				XV_Y,			40,
                PANEL_ITEM_MENU,     	fontsizemenu ,
                NULL) ;

	BBoard[grp_id].infotxt = 
			(Panel_item)xv_create(BBoard[grp_id].bb_panel, PANEL_TEXT,
					PANEL_LABEL_X,	10,
					PANEL_LABEL_Y,	70,
					PANEL_LABEL_STRING, NULL,
					PANEL_VALUE, "Select one of the above objects",
					PANEL_VALUE_DISPLAY_LENGTH,
							strlen("Select one of the above objects"),
					PANEL_VALUE_UNDERLINED,	FALSE,
					PANEL_READ_ONLY,	TRUE,
					NULL) ;

	BBoard[grp_id].object_indicator =
			(Panel_item)xv_create(BBoard[grp_id].bb_panel, PANEL_TEXT,
				PANEL_LABEL_X,	480,
				PANEL_LABEL_Y,	5,
				PANEL_LABEL_STRING, "Current Mode:",
				PANEL_VALUE_X,	500,
				PANEL_VALUE_Y,	20,
				PANEL_VALUE, "NONE",
				PANEL_VALUE_UNDERLINED,	FALSE,
				PANEL_READ_ONLY,	TRUE,
				NULL) ;

	xwin = (Window) xv_get(xv_get( BBoard[grp_id].bb_panel, 
							CANVAS_NTH_PAINT_WINDOW, 0), XV_XID) ;
	XDefineCursor( theDisplay, xwin, hand1) ;
	XFlush(theDisplay) ;

	(void) xv_set(canvas_paint_window( BBoard[grp_id].bb_canvas),
		WIN_EVENT_PROC,	bb_event_proc,
		WIN_CONSUME_EVENTS,	KBD_DONE, KBD_USE, LOC_DRAG, LOC_MOVE,
				LOC_WINENTER, LOC_WINEXIT, WIN_ASCII_EVENTS, 
				WIN_MOUSE_BUTTONS, NULL,
		NULL) ;

	
	xwin = xv_get( canvas_paint_window( BBoard[grp_id].bb_canvas), XV_XID) ;

	/*Create other cursors fro blackboard */
	top_left = XCreateFontCursor( theDisplay, XC_top_left_corner) ;
	bottom_right = XCreateFontCursor( theDisplay, XC_bottom_right_corner) ;
	circle_cursor = XCreateFontCursor( theDisplay, XC_circle) ;


	/* Creating the GCs and pixmaps reqd . */
	BBoard[grp_id].bb_pixmap = XCreatePixmap(theDisplay, DefaultRootWindow(theDisplay), 
					HeightOfScreen(theScreen), WidthOfScreen(theScreen), 
					XDefaultDepth(theDisplay,XScreenNumberOfScreen(theScreen))) ; 
	gcvalues.graphics_exposures = False ;
	gcvalues.background = 0 ; /*WORKHERE*/
	gcvalues.foreground = 1 ;


/**	
	BBoard[grp_id].bb_gc= XCreateGC(theDisplay, BBoard[grp_id].bb_pixmap, 
							GCForeground|GCBackground|GCGraphicsExposures, 
							&gcvalues) ;
**/
	BBoard[grp_id].bb_gc= XCreateGC(theDisplay, BBoard[grp_id].bb_pixmap, 
							0, NULL) ;
	BBoard[grp_id].bb_bgc= XCreateGC(theDisplay, xwin, 0, NULL) ;
	XSetForeground( theDisplay, BBoard[grp_id].bb_gc, Black) ;
	XSetBackground( theDisplay, BBoard[grp_id].bb_gc, White);
	XSetForeground( theDisplay, BBoard[grp_id].bb_bgc, Black) ;
	XSetBackground( theDisplay, BBoard[grp_id].bb_bgc, White) ;
	Fill_Style[grp_id] = NONE ;
	Line_Style[grp_id] = LineSolid ;
	Line_Width[grp_id] = 1 ;

	/* The pixmap has garbage value. Clear it first */
	XSetFunction( theDisplay, BBoard[grp_id].bb_gc, GXcopy) ;
	XSetForeground( theDisplay, BBoard[grp_id].bb_gc, White) ;
	XSetBackground( theDisplay, BBoard[grp_id].bb_gc, Black) ;
	XFillRectangle(theDisplay, BBoard[grp_id].bb_pixmap, BBoard[grp_id].bb_gc, 0,
					0, HeightOfScreen(theScreen), WidthOfScreen(theScreen) );
	XSetForeground( theDisplay, BBoard[grp_id].bb_gc, Black) ;
	XSetBackground( theDisplay, BBoard[grp_id].bb_gc, White) ;

	/*Create the erasetile */
	erasetile = XCreatePixmapFromBitmapData( theDisplay, xwin,
					fill_images[0], 8, 8, 1, 0, 1) ;

	/*Create default fill_tile */
	fill_tile[grp_id] = XCreatePixmapFromBitmapData(theDisplay, xwin,
					fill_images[Fill_Style[grp_id]], 8, 8, 1, 0, 1) ;
	/* Set the Font style for the gc */
	font = (Xv_font) xv_find( BBoard[grp_id].bb_frame, FONT,
						FONT_FAMILY,	FONT_FAMILY_DEFAULT,
						FONT_STYLE,		FONT_STYLE_DEFAULT,
						FONT_SCALE,		WIN_SCALE_MEDIUM,
						FONT_SIZES_FOR_SCALE,	12,14,16,22,
						NULL) ;
	if ( font != NULL)
	{
		cur_font[grp_id] = (XFontStruct *)xv_get(font, FONT_INFO) ;
		cur_char_width[grp_id] = cur_font[grp_id]->max_bounds.rbearing
									- cur_font[grp_id]->min_bounds.lbearing ;
		cur_char_height[grp_id] = cur_font[grp_id]->max_bounds.ascent 
									+ cur_font[grp_id]->max_bounds.descent ;
		XSetFont( theDisplay, BBoard[grp_id].bb_gc, cur_font[grp_id]->fid) ;
		XSetFont( theDisplay, BBoard[grp_id].bb_bgc, cur_font[grp_id]->fid) ;
	}

	/* Set in XOR mode */
	XSetFunction( theDisplay, BBoard[grp_id].bb_gc, GXxor) ;
	XSetFunction( theDisplay, BBoard[grp_id].bb_bgc, GXxor) ;

	/* Set the line attr */
	XSetLineAttributes( theDisplay, BBoard[grp_id].bb_bgc, 1,
			LineOnOffDash, CapProjecting, JoinMiter) ;
	dash_list[0]=1;dash_list[1]=2 ;
	XSetDashes( theDisplay , BBoard[grp_id].bb_bgc, 0, dash_list, 2) ;

	xv_set(BBoard[grp_id].bb_canvas, XV_SHOW, TRUE, NULL ) ;
	xv_set(BBoard[grp_id].bb_panel, XV_SHOW, TRUE, NULL ) ;
	xv_set(BBoard[grp_id].infotxt, XV_SHOW, TRUE, NULL ) ;
	xv_set(BBoard[grp_id].bb_frame, XV_SHOW, TRUE, NULL ) ;

}


/*------------------------------------------------bb_event_proc()-------------*/
void	bb_event_proc( window, event)
Xv_Window	window ;
Event	*event ;
{
	Window	win ;
	int		grp_id ;
	short	piv_x, piv_y, b, h ;
	short	px, py ;
	int	i;
	short	x,y ;
	XPoint	xpts[30]; 
	char	c ;

	XColor	rgb, hwcol ;
	int		st ;

	win = xv_get(window, XV_XID) ;

	grp_id = find_group_from_bbcanvas( event) ;

	/* For Ascii events ??*/
	if (event_is_ascii(event))
	{
		if (event_is_up(event))
					return ; /* ???? */
		if ((CurrBBoardObj[grp_id]==TEXT) && ( InTextMode[grp_id] == TRUE))
		{
			c = (char) event_action(event) ;

			if ( c == ENTER )
			{
				/*Reset the marker*/
				reset_marker(grp_id) ;
				/* get new pivot point and current pos */
				if (BBoardObj[grp_id].piv_y+cur_char_height[grp_id] 
						< BBFRM_HEIGHT)
				{
					BBoardObj[grp_id].piv_y += cur_char_height[grp_id] ;
					BBoardObj[grp_id].current_x = BBoardObj[grp_id].piv_x ;
					BBoardObj[grp_id].current_y += cur_char_height[grp_id] ;
				}
				/*Draw marker */
				reset_marker(grp_id) ;
				return ;/*WORKHERE*/	
			}

			if (( c == BACKSPACE) || ( c == DELETE) )
			{
				if ( BBoardObj[grp_id].current_x 
							>= BBoardObj[grp_id].piv_x+cur_char_width[grp_id] )
				{
					/* The marker reset*/
					reset_marker( grp_id) ;

					XSetFunction( theDisplay, BBoard[grp_id].bb_bgc, GXcopy) ;
					XSetForeground( theDisplay, BBoard[grp_id].bb_bgc, White) ;
					XSetBackground( theDisplay, BBoard[grp_id].bb_bgc, Black) ;
					XSetFunction( theDisplay, BBoard[grp_id].bb_gc, GXcopy) ;
					XSetForeground( theDisplay, BBoard[grp_id].bb_gc, White) ;
					XSetBackground( theDisplay, BBoard[grp_id].bb_gc, Black) ;
					XFillRectangle(theDisplay, win, BBoard[grp_id].bb_bgc, 
							BBoardObj[grp_id].current_x
								-cur_char_width[grp_id]
								-cur_font[grp_id]->max_bounds.lbearing,
							BBoardObj[grp_id].current_y
								-cur_font[grp_id]->max_bounds.ascent,
							cur_char_width[grp_id], cur_char_height[grp_id]) ;

					XFillRectangle(theDisplay, BBoard[grp_id].bb_pixmap, 
							BBoard[grp_id].bb_gc, 
							BBoardObj[grp_id].current_x
								-cur_char_width[grp_id]
								-cur_font[grp_id]->max_bounds.lbearing,
							BBoardObj[grp_id].current_y
								-cur_font[grp_id]->max_bounds.ascent,
							cur_char_width[grp_id], cur_char_height[grp_id]) ;
					XSetForeground( theDisplay, BBoard[grp_id].bb_gc, Black) ;
					XSetBackground( theDisplay, BBoard[grp_id].bb_gc, White) ;
					XSetForeground( theDisplay, BBoard[grp_id].bb_bgc, Black) ;
					XSetBackground( theDisplay, BBoard[grp_id].bb_bgc, White) ;

					/*send the char */
					T_send_BBoardChar(grp_id, c,Font_Style[grp_id],
						Font_Family[grp_id], Font_Size[grp_id], 
						Curr_Color[grp_id],
						BBoardObj[grp_id].current_x, 
						BBoardObj[grp_id].current_y) ;

					/*Go back one pos */
					BBoardObj[grp_id].current_x -= cur_char_width[grp_id] ;	

					/*Draw marker*/
					reset_marker ( grp_id) ;
				}
			}
						

			if (( isalnum(c) )||(isspace(c)) ||(ispunct(c)))
			{

XSetForeground( theDisplay, BBoard[grp_id].bb_gc, col_pixel[Curr_Color[grp_id]]) ;
XSetForeground( theDisplay, BBoard[grp_id].bb_bgc, col_pixel[Curr_Color[grp_id]]) ;
			XDrawString(theDisplay, BBoard[grp_id].bb_pixmap, BBoard[grp_id].bb_gc, 
				BBoardObj[grp_id].current_x, BBoardObj[grp_id].current_y, 
				&c, 1) ; 
			XDrawString(theDisplay, win, BBoard[grp_id].bb_bgc, 
				BBoardObj[grp_id].current_x, BBoardObj[grp_id].current_y, 
				&c, 1) ; 

			/*Draw marker*/
			reset_marker ( grp_id) ;

			/*send the char */
			T_send_BBoardChar(grp_id, c,Font_Style[grp_id],
				Font_Family[grp_id], Font_Size[grp_id], 
				Curr_Color[grp_id] ,
				BBoardObj[grp_id].current_x, BBoardObj[grp_id].current_y) ;

			/*Advance to next pos */
			if ( BBoardObj[grp_id].current_x + 2*cur_char_width[grp_id]
					> BBFRM_WIDTH)
			{
				/*Goto next line */
				if (BBoardObj[grp_id].piv_y+cur_char_height[grp_id] 
						< BBFRM_HEIGHT)
				{
					BBoardObj[grp_id].piv_y += cur_char_height[grp_id] ;
					BBoardObj[grp_id].current_x = BBoardObj[grp_id].piv_x ;
					BBoardObj[grp_id].current_y += cur_char_height[grp_id] ;
				}
			}
			else
				BBoardObj[grp_id].current_x += cur_char_width[grp_id] ;	
			/* The marker reset*/
			reset_marker(grp_id) ;

			}
		}
	}
	else
	{
		switch(event_action(event))
		{

			case LOC_MOVE:
			case LOC_DRAG:
			{
				if ( DrawingOn[grp_id])
				{
			
/* The line style and fill style are reset while expanding the figure */
				XSetFillStyle( theDisplay, BBoard[grp_id].bb_gc, FillSolid) ;
				XSetFillStyle( theDisplay, BBoard[grp_id].bb_bgc, FillSolid) ;
				XSetLineAttributes( theDisplay, BBoard[grp_id].bb_gc, 1, 
						Line_Style[grp_id], CapProjecting, JoinMiter) ;
				XSetLineAttributes( theDisplay, BBoard[grp_id].bb_bgc, 1, 
						Line_Style[grp_id], CapProjecting, JoinMiter) ;

				switch(CurrBBoardObj[grp_id])
				{
					case CIRC:
					case RECT:
					case ERASE:
						calculate_obj_size( grp_id, BBoardObj[grp_id].current_x,
							BBoardObj[grp_id].current_y,BBoardObj[grp_id].piv_x,
							BBoardObj[grp_id].piv_y, &piv_x, &piv_y, &b, &h) ;
						if ( CurrBBoardObj[grp_id] == CIRC)
						{
							XDrawArc( theDisplay, BBoard[grp_id].bb_pixmap, 
								BBoard[grp_id].bb_gc, 
								piv_x, piv_y, b, h, 0, 64*360) ;
							XDrawArc( theDisplay, win, BBoard[grp_id].bb_bgc, 
								piv_x, piv_y, b, h, 0, 64*360) ;
						}
						else
						{
							XDrawRectangle( theDisplay, BBoard[grp_id].bb_pixmap, 
								BBoard[grp_id].bb_gc,
								piv_x, piv_y, b, h) ;
							XDrawRectangle( theDisplay, win, BBoard[grp_id].bb_bgc,
								piv_x, piv_y, b, h) ;
						}

						/* Get the current position */
						BBoardObj[grp_id].current_x = event_x(event) ;
						BBoardObj[grp_id].current_y = event_y(event) ;

						calculate_obj_size( grp_id, BBoardObj[grp_id].current_x,
							BBoardObj[grp_id].current_y,BBoardObj[grp_id].piv_x,
							BBoardObj[grp_id].piv_y, &piv_x, &piv_y, &b, &h) ;
						if ( CurrBBoardObj[grp_id] == CIRC)
						{
							XDrawArc( theDisplay, BBoard[grp_id].bb_pixmap, 
								BBoard[grp_id].bb_gc, 
								piv_x, piv_y, b, h, 0, 64*360) ;
							XDrawArc( theDisplay, win, BBoard[grp_id].bb_bgc, 
								piv_x, piv_y, b, h, 0, 64*360) ;
						}
						else
						{
							XDrawRectangle( theDisplay, BBoard[grp_id].bb_pixmap, 
								BBoard[grp_id].bb_gc,
								piv_x, piv_y, b, h) ;
							XDrawRectangle( theDisplay, win, BBoard[grp_id].bb_bgc,
								piv_x, piv_y, b, h) ;
						}
						break ;

					case ARC:

						XDrawLine(theDisplay, BBoard[grp_id].bb_pixmap,  
							BBoard[grp_id].bb_gc, 
							BBoardObj[grp_id].piv_x, BBoardObj[grp_id].piv_y, 
							BBoardObj[grp_id].current_x, 
							BBoardObj[grp_id].current_y) ;
						XDrawLine(theDisplay, win,  BBoard[grp_id].bb_bgc, 
							BBoardObj[grp_id].piv_x, BBoardObj[grp_id].piv_y, 
							BBoardObj[grp_id].current_x, 
							BBoardObj[grp_id].current_y) ;

						BBoardObj[grp_id].current_x = event_x(event) ;
						BBoardObj[grp_id].current_y = event_y(event) ;
						XDrawLine(theDisplay, BBoard[grp_id].bb_pixmap, 
							BBoard[grp_id].bb_gc, 
							BBoardObj[grp_id].piv_x, BBoardObj[grp_id].piv_y, 
							BBoardObj[grp_id].current_x, 
							BBoardObj[grp_id].current_y) ;
						XDrawLine(theDisplay, win, BBoard[grp_id].bb_bgc, 
							BBoardObj[grp_id].piv_x, BBoardObj[grp_id].piv_y, 
							BBoardObj[grp_id].current_x, 
							BBoardObj[grp_id].current_y) ;
						break ;
						
					case POLYLINE:
					case POLYGON:

/*XSetFunction(theDisplay, BBoard[grp_id].bb_bgc, GXcopy) ;*/
/*XSetFunction(theDisplay, BBoard[grp_id].bb_gc, GXcopy) ;*/
/*XSetForeground( theDisplay, BBoard[grp_id].bb_bgc, White);*/
/*XSetForeground( theDisplay, BBoard[grp_id].bb_gc, White) ;*/
/*XSetBackground( theDisplay, BBoard[grp_id].bb_bgc, col_pixel[Curr_Color[grp_id]]);*/
/*XSetBackground( theDisplay, BBoard[grp_id].bb_gc, col_pixel[Curr_Color[grp_id]]) ;*/
						XDrawLine(theDisplay, BBoard[grp_id].bb_pixmap,  
							BBoard[grp_id].bb_gc, 
							BBoardObj[grp_id].piv_x, BBoardObj[grp_id].piv_y, 
							BBoardObj[grp_id].current_x, 
							BBoardObj[grp_id].current_y) ;

						XDrawLine(theDisplay, win,  BBoard[grp_id].bb_bgc, 
							BBoardObj[grp_id].piv_x, BBoardObj[grp_id].piv_y, 
							BBoardObj[grp_id].current_x, 
							BBoardObj[grp_id].current_y) ;
XFlush(theDisplay) ;
						BBoardObj[grp_id].current_x = event_x(event) ;
						BBoardObj[grp_id].current_y = event_y(event) ;

/*XSetBackground( theDisplay, BBoard[grp_id].bb_bgc, White);*/
/*XSetBackground( theDisplay, BBoard[grp_id].bb_gc, White) ;*/
/*XSetForeground( theDisplay, BBoard[grp_id].bb_bgc, col_pixel[Curr_Color[grp_id]]);*/
/*XSetForeground( theDisplay, BBoard[grp_id].bb_gc, col_pixel[Curr_Color[grp_id]]) ;*/
						XDrawLine(theDisplay, BBoard[grp_id].bb_pixmap, 
							BBoard[grp_id].bb_gc, 
							BBoardObj[grp_id].piv_x, BBoardObj[grp_id].piv_y, 
							BBoardObj[grp_id].current_x, 
							BBoardObj[grp_id].current_y) ;
						XDrawLine(theDisplay, win, BBoard[grp_id].bb_bgc, 
							BBoardObj[grp_id].piv_x, BBoardObj[grp_id].piv_y, 
							BBoardObj[grp_id].current_x, 
							BBoardObj[grp_id].current_y) ;
						break ;
						

					case NONE:
					default:
						break ;
				}
				}
				break ;
			}

			case ACTION_SELECT:
			case MS_LEFT:
			{
				if (event_is_up(event))
					break ;
				XSetFillStyle( theDisplay, BBoard[grp_id].bb_gc, FillSolid) ;
				XSetFillStyle( theDisplay, BBoard[grp_id].bb_bgc, FillSolid) ;
				XSetLineAttributes( theDisplay, BBoard[grp_id].bb_gc, 1, 
						Line_Style[grp_id], CapProjecting, JoinMiter) ;
				XSetLineAttributes( theDisplay, BBoard[grp_id].bb_bgc, 1, 
						Line_Style[grp_id], CapProjecting, JoinMiter) ;
XSetForeground(theDisplay, BBoard[grp_id].bb_gc, Black) ;
XSetForeground(theDisplay, BBoard[grp_id].bb_bgc, Black) ;
				switch(CurrBBoardObj[grp_id])
				{
					case TEXT:
						if ( InTextMode[grp_id] == TRUE)
						{
						/*Reset the marker */
							reset_marker(grp_id) ;
						}
						BBoardObj[grp_id].piv_x = event_x(event) ;
						BBoardObj[grp_id].piv_y = event_y(event) ;
						BBoardObj[grp_id].current_x = event_x(event) ;
						BBoardObj[grp_id].current_y = event_y(event) ;
						InTextMode[grp_id] = TRUE ;

						/*Draw Marker */
						reset_marker(grp_id) ;

						break ;
					case CIRC:
					case RECT:
					case	ERASE:
						if ( DrawingOn[grp_id] == FALSE)
						{
						BBoardObj[grp_id].current_x = event_x(event) ;
						BBoardObj[grp_id].current_y = event_y(event) ;
						BBoardObj[grp_id].piv_x = event_x(event) ;
						BBoardObj[grp_id].piv_y = event_y(event) ;
						calculate_obj_size( grp_id, BBoardObj[grp_id].current_x,
							BBoardObj[grp_id].current_y, 
							BBoardObj[grp_id].piv_x,
							BBoardObj[grp_id].piv_y, &piv_x, &piv_y, &b, &h) ;
						if ( CurrBBoardObj[grp_id] == CIRC)
						{
							XDrawArc( theDisplay, BBoard[grp_id].bb_pixmap, 
									BBoard[grp_id].bb_gc, 
									piv_x, piv_y, b, h, 0, 64*360) ;
							XDrawArc( theDisplay, win, BBoard[grp_id].bb_bgc, 
									piv_x, piv_y, b, h, 0, 64*360) ;
						}
						else
						{
							XDrawRectangle( theDisplay, BBoard[grp_id].bb_pixmap, 
								BBoard[grp_id].bb_gc,
								piv_x, piv_y, b, h) ;
							XDrawRectangle( theDisplay, win, BBoard[grp_id].bb_bgc,
								piv_x, piv_y, b, h) ;
							if ( CurrBBoardObj[grp_id] == RECT)
								XDefineCursor( theDisplay, win, bottom_right) ;
						}
						BBoardObj[grp_id].points[
									2*BBoardObj[grp_id].current_point]
                                        = BBoardObj[grp_id].current_x ;
						BBoardObj[grp_id].points[
									2*BBoardObj[grp_id].current_point+1]
                                        = BBoardObj[grp_id].current_y ;
						BBoardObj[grp_id].current_point ++ ;
                        if (BBoardObj[grp_id].current_point > MAX_PTS)
                        {
                            DrawingOn[grp_id] = FALSE ;
							xv_set(BBoard[grp_id].infotxt,
								PANEL_VALUE, "Select one of the above objects",
								PANEL_VALUE_DISPLAY_LENGTH, 32, NULL) ;
                            BBoardObj[grp_id].current_point = 0 ;
                        }
						DrawingOn[grp_id] = TRUE ;
						}
						break ;
					case ARC:

						if ((BBoardObj[grp_id].current_point == 0) ||
							(BBoardObj[grp_id].current_point == 1))
						{
							
							BBoardObj[grp_id].current_x = event_x(event) ;
							BBoardObj[grp_id].current_y = event_y(event) ;
							BBoardObj[grp_id].piv_x = event_x(event) ;
							BBoardObj[grp_id].piv_y = event_y(event) ;
	
							XDrawLine(theDisplay, BBoard[grp_id].bb_pixmap, 
								BBoard[grp_id].bb_gc, 
								BBoardObj[grp_id].piv_x, 
								BBoardObj[grp_id].piv_y, 
								BBoardObj[grp_id].current_x, 
								BBoardObj[grp_id].current_y) ;
							XDrawLine(theDisplay, win, BBoard[grp_id].bb_bgc, 
								BBoardObj[grp_id].piv_x, 
								BBoardObj[grp_id].piv_y, 
								BBoardObj[grp_id].current_x, 
								BBoardObj[grp_id].current_y) ;


							BBoardObj[grp_id].points[
									2*BBoardObj[grp_id].current_point]
										= BBoardObj[grp_id].current_x ;
							BBoardObj[grp_id].points[
									2*BBoardObj[grp_id].current_point + 1]
										= BBoardObj[grp_id].current_y ;
							BBoardObj[grp_id].current_point ++ ;
							DrawingOn[grp_id] = TRUE ;
						}
						else
						{
							BBoardObj[grp_id].points[	
									2*BBoardObj[grp_id].current_point]
										= BBoardObj[grp_id].current_x ;
							BBoardObj[grp_id].points[
									2*BBoardObj[grp_id].current_point + 1]
										= BBoardObj[grp_id].current_y ;
						
							/*Erase the lines drawn*/
							for ( i=0;i<3; i+=2)
							{
								XDrawLine( theDisplay, BBoard[grp_id].bb_pixmap, 
									BBoard[grp_id].bb_gc,
									BBoardObj[grp_id].points[i],
									BBoardObj[grp_id].points[i+1],
									BBoardObj[grp_id].points[i+2],
									BBoardObj[grp_id].points[i+3]) ;
								XDrawLine( theDisplay, win, BBoard[grp_id].bb_bgc,
									BBoardObj[grp_id].points[i],
									BBoardObj[grp_id].points[i+1],
									BBoardObj[grp_id].points[i+2],
									BBoardObj[grp_id].points[i+3]) ;
							}
							/* Draw the ARC here */
							
							BBoardObj[grp_id].current_point = 0 ;
							DrawingOn[grp_id] = FALSE ;
							xv_set(BBoard[grp_id].infotxt,
								PANEL_VALUE, "Select one of the above objects",
								PANEL_VALUE_DISPLAY_LENGTH, 32, NULL) ;
						}
						break ;
							
					case POLYLINE:
					case POLYGON:

							BBoardObj[grp_id].current_x = event_x(event) ;
							BBoardObj[grp_id].current_y = event_y(event) ;
							BBoardObj[grp_id].piv_x = event_x(event) ;
							BBoardObj[grp_id].piv_y = event_y(event) ;
	
							XDrawLine(theDisplay, BBoard[grp_id].bb_pixmap, 
								BBoard[grp_id].bb_gc, 
								BBoardObj[grp_id].piv_x, 
								BBoardObj[grp_id].piv_y, 
								BBoardObj[grp_id].current_x, 
								BBoardObj[grp_id].current_y) ;

							XDrawLine(theDisplay, win, BBoard[grp_id].bb_bgc, 
								BBoardObj[grp_id].piv_x, 
								BBoardObj[grp_id].piv_y, 
								BBoardObj[grp_id].current_x, 
								BBoardObj[grp_id].current_y) ;
							BBoardObj[grp_id].points[
								2*BBoardObj[grp_id].current_point]
										= BBoardObj[grp_id].current_x ;
							BBoardObj[grp_id].points[
								2*BBoardObj[grp_id].current_point+1]
										= BBoardObj[grp_id].current_y ;
							BBoardObj[grp_id].current_point ++ ;
							DrawingOn[grp_id] = TRUE ;
							if (BBoardObj[grp_id].current_point > MAX_PTS)
							{
								DrawingOn[grp_id] = FALSE ;
								BBoardObj[grp_id].current_point = 0 ;
							}
							break ;
							
					case NONE:
					default :
						break ;
				}
			}
			break ;

			case ACTION_ADJUST:
			case MS_MIDDLE:
			{
				if ( DrawingOn[grp_id])
				{
				BBoardObj[grp_id].points[2*BBoardObj[grp_id].current_point]
									= event_x(event) ;
				BBoardObj[grp_id].points[2*BBoardObj[grp_id].current_point+1]
									= event_y(event) ;
				BBoardObj[grp_id].current_point ++ ;
				XSetFunction( theDisplay, BBoard[grp_id].bb_gc, GXor);/*OR*/ 
				XSetFunction( theDisplay, BBoard[grp_id].bb_bgc, GXor);/*OR*/ 
				if ( Fill_Style[grp_id] != NONE)
				{
					XSetFillStyle( theDisplay, BBoard[grp_id].bb_gc, TILED) ;
					XSetFillStyle( theDisplay, BBoard[grp_id].bb_bgc, TILED) ;
					XSetTile(theDisplay, BBoard[grp_id].bb_gc, fill_tile[grp_id]) ;
					XSetTile(theDisplay, BBoard[grp_id].bb_bgc, fill_tile[grp_id]) ;
				}

				XSetLineAttributes(theDisplay,BBoard[grp_id].bb_gc, Line_Width[grp_id],
					Line_Style[grp_id], CapProjecting, JoinMiter) ;
				XSetLineAttributes(theDisplay,BBoard[grp_id].bb_bgc, 
					Line_Width[grp_id],
					Line_Style[grp_id], CapProjecting, JoinMiter) ;

				switch(CurrBBoardObj[grp_id])
				{
					case TEXT:
						break ;
					case ARC:
						break ;
					case ERASE:
						calculate_obj_size(grp_id, BBoardObj[grp_id].points[0],
							BBoardObj[grp_id].points[1], 
							BBoardObj[grp_id].points[2],	
							BBoardObj[grp_id].points[3], &px, &py, &b, &h) ;

						XSetFunction( theDisplay, BBoard[grp_id].bb_bgc, GXcopy) ;
						XSetForeground( theDisplay, BBoard[grp_id].bb_bgc, White) ;
						XSetBackground( theDisplay, BBoard[grp_id].bb_bgc, col_pixel[Curr_Color[grp_id]]) ;

						XSetFunction( theDisplay, BBoard[grp_id].bb_gc, GXcopy) ;
						XSetForeground( theDisplay, BBoard[grp_id].bb_gc, White) ;
						XSetBackground( theDisplay, BBoard[grp_id].bb_gc, col_pixel[Curr_Color[grp_id]]) ;
						/*Set the erase fill style */
						XSetFillStyle( theDisplay, BBoard[grp_id].bb_gc, TILED) ;
						XSetFillStyle( theDisplay, BBoard[grp_id].bb_bgc, TILED) ;
						XSetTile(theDisplay, BBoard[grp_id].bb_gc, erasetile) ;
						XSetTile(theDisplay, BBoard[grp_id].bb_bgc, erasetile) ;
						XFillRectangle(theDisplay, win, BBoard[grp_id].bb_bgc, 
								px, py, b+1, h+1) ;
	
						XFillRectangle(theDisplay, BBoard[grp_id].bb_pixmap, 
								BBoard[grp_id].bb_gc,
								px, py, b+1, h+1) ;
						XSetForeground( theDisplay, BBoard[grp_id].bb_gc, col_pixel[Curr_Color[grp_id]]) ;
						XSetBackground( theDisplay, BBoard[grp_id].bb_gc, White) ;
						XSetForeground( theDisplay, BBoard[grp_id].bb_bgc, col_pixel[Curr_Color[grp_id]]) ;
						XSetBackground( theDisplay, BBoard[grp_id].bb_bgc, White) ;
						XSetFunction( theDisplay, BBoard[grp_id].bb_bgc, GXxor) ;
						/*Reset the fill style */
						XSetFillStyle( theDisplay, BBoard[grp_id].bb_gc, TILED) ;
						XSetFillStyle( theDisplay, BBoard[grp_id].bb_bgc, TILED) ;
						XSetTile(theDisplay, BBoard[grp_id].bb_gc, fill_tile[grp_id]) ;
						XSetTile(theDisplay, BBoard[grp_id].bb_bgc, 
											fill_tile[grp_id]) ;
	
						/*Refresh the grid if on */
						if (GRID[grp_id] == ON)
							draw_grid(grp_id) ;
							
						break ;

					case CIRC:
						calculate_obj_size(grp_id, BBoardObj[grp_id].points[0],
							BBoardObj[grp_id].points[1], 
							BBoardObj[grp_id].points[2],	
							BBoardObj[grp_id].points[3], &px, &py, &b, &h) ;
						if ( Fill_Style[grp_id] != NONE)
						{
							XFillArc(theDisplay, BBoard[grp_id].bb_pixmap, 
								BBoard[grp_id].bb_gc, px+1, py+1,
								b-1, h-1, 0, 64*360) ;
							XFillArc(theDisplay, win, BBoard[grp_id].bb_bgc, px+1, 
								py+1, b-1, h-1, 0, 64*360) ;
						}
						XSetFillStyle( theDisplay, BBoard[grp_id].bb_gc, FillSolid) ;
						XSetFillStyle( theDisplay, BBoard[grp_id].bb_bgc, FillSolid) ;
						XSetLineAttributes( theDisplay, BBoard[grp_id].bb_gc, 
							Line_Width[grp_id], Line_Style[grp_id], 
							CapProjecting, JoinMiter) ;
						XSetLineAttributes( theDisplay, BBoard[grp_id].bb_bgc, 
							Line_Width[grp_id], Line_Style[grp_id], 
							CapProjecting, JoinMiter) ;
XSetForeground(theDisplay, BBoard[grp_id].bb_gc, col_pixel[Curr_Color[grp_id]]) ;
XSetForeground(theDisplay, BBoard[grp_id].bb_bgc, col_pixel[Curr_Color[grp_id]]) ;
						XDrawArc(theDisplay, BBoard[grp_id].bb_pixmap, 
							BBoard[grp_id].bb_gc, px, py, b, h, 0, 64*360) ;
						XDrawArc(theDisplay, win, BBoard[grp_id].bb_bgc, px, py, b, h,
							0, 64*360) ;
XSetForeground(theDisplay, BBoard[grp_id].bb_gc, Black) ;
XSetForeground(theDisplay, BBoard[grp_id].bb_bgc, Black) ;
						break ;

					case RECT:

						XSetFunction( theDisplay, BBoard[grp_id].bb_gc, GXor);/*OR*/ 
						XSetFunction( theDisplay, BBoard[grp_id].bb_bgc, GXor);/*OR*/ 
						calculate_obj_size(grp_id, BBoardObj[grp_id].points[0],
							BBoardObj[grp_id].points[1], 
							BBoardObj[grp_id].points[2],	
							BBoardObj[grp_id].points[3], &px, &py, &b, &h) ;
						if ( Fill_Style[grp_id] != NONE)
						{
							XFillRectangle(theDisplay, BBoard[grp_id].bb_pixmap, 
								BBoard[grp_id].bb_gc, px+1,
								py+1, b-1, h-1) ;
							XFillRectangle(theDisplay, win, BBoard[grp_id].bb_bgc, 
								px+1, py+1, b-1, h-1) ;
						}
						XSetFillStyle( theDisplay, BBoard[grp_id].bb_gc, FillSolid) ;
						XSetFillStyle( theDisplay, BBoard[grp_id].bb_bgc, FillSolid) ;
						XSetLineAttributes( theDisplay, BBoard[grp_id].bb_gc, 
							Line_Width[grp_id], Line_Style[grp_id], 
							CapProjecting, JoinMiter) ;
						XSetLineAttributes( theDisplay, BBoard[grp_id].bb_bgc, 
							Line_Width[grp_id], Line_Style[grp_id], 
							CapProjecting, JoinMiter) ;
XSetForeground(theDisplay, BBoard[grp_id].bb_gc, col_pixel[Curr_Color[grp_id]]) ;
XSetForeground(theDisplay, BBoard[grp_id].bb_bgc, col_pixel[Curr_Color[grp_id]]) ;
						XDrawRectangle(theDisplay, BBoard[grp_id].bb_pixmap, 
							BBoard[grp_id].bb_gc, px, py, b, h) ;
						XDrawRectangle(theDisplay, win, BBoard[grp_id].bb_bgc, px, py, 
							b, h) ;
						XDefineCursor( theDisplay, win, top_left) ;
XSetForeground(theDisplay, BBoard[grp_id].bb_gc, Black) ;
XSetForeground(theDisplay, BBoard[grp_id].bb_bgc, Black) ;
						break ;

					case POLYLINE:
						XSetFillStyle( theDisplay, BBoard[grp_id].bb_gc, FillSolid) ;
						XSetFillStyle( theDisplay, BBoard[grp_id].bb_bgc, FillSolid) ;
						XSetLineAttributes(theDisplay,BBoard[grp_id].bb_gc, 
							Line_Width[grp_id], Line_Style[grp_id], 
							CapProjecting, JoinMiter) ;
						XSetLineAttributes(theDisplay,BBoard[grp_id].bb_bgc, 
							Line_Width[grp_id], Line_Style[grp_id], 
							CapProjecting, JoinMiter) ;
XSetForeground(theDisplay, BBoard[grp_id].bb_gc, col_pixel[Curr_Color[grp_id]]) ;
XSetForeground(theDisplay, BBoard[grp_id].bb_bgc, col_pixel[Curr_Color[grp_id]]) ;
						for(i=0;i<BBoardObj[grp_id].current_point-1; i++)
						{
							XDrawLine(theDisplay, BBoard[grp_id].bb_pixmap, 
								BBoard[grp_id].bb_gc, 
								BBoardObj[grp_id].points[2*i], 
								BBoardObj[grp_id].points[2*i+1], 
								BBoardObj[grp_id].points[2*i+2], 
								BBoardObj[grp_id].points[2*i+3]) ;
							XDrawLine(theDisplay, win, BBoard[grp_id].bb_bgc, 
								BBoardObj[grp_id].points[2*i], 
								BBoardObj[grp_id].points[2*i+1], 
								BBoardObj[grp_id].points[2*i+2], 
								BBoardObj[grp_id].points[2*i+3]) ;
						}
XSetForeground(theDisplay, BBoard[grp_id].bb_gc, Black) ;
XSetForeground(theDisplay, BBoard[grp_id].bb_bgc, Black) ;

						break ;
						
					case POLYGON:

						/* Fill the XPoint structure with points */
						fill_polypoints(xpts, grp_id) ;
 
						/* Fill the polygon*/
						XSetFunction( theDisplay, BBoard[grp_id].bb_gc, GXor);/*OR*/ 
						XSetFunction( theDisplay, BBoard[grp_id].bb_bgc, GXor);/*OR*/ 
XSetForeground(theDisplay, BBoard[grp_id].bb_gc, col_pixel[Curr_Color[grp_id]]) ;
XSetForeground(theDisplay, BBoard[grp_id].bb_bgc, col_pixel[Curr_Color[grp_id]]) ;
						if ( Fill_Style[grp_id] != NONE)
						{
							XFillPolygon ( theDisplay, BBoard[grp_id].bb_pixmap, 
								BBoard[grp_id].bb_gc,
								(XPoint *)xpts, 
								(int)(BBoardObj[grp_id].current_point), 
								Complex, CoordModeOrigin) ;
							XFillPolygon ( theDisplay, win, BBoard[grp_id].bb_bgc,
								(XPoint *)xpts, 
								(int)(BBoardObj[grp_id].current_point), 
								Complex, CoordModeOrigin) ;
						}

						XSetFillStyle( theDisplay, BBoard[grp_id].bb_gc, FillSolid) ;
						XSetFillStyle( theDisplay, BBoard[grp_id].bb_bgc, FillSolid) ;
						XSetLineAttributes( theDisplay, BBoard[grp_id].bb_gc, 
							Line_Width[grp_id], Line_Style[grp_id], 
							CapProjecting, JoinMiter) ;
						XSetLineAttributes( theDisplay, BBoard[grp_id].bb_bgc, 
							Line_Width[grp_id], Line_Style[grp_id], 
							CapProjecting, JoinMiter) ;

						for(i=0;i<BBoardObj[grp_id].current_point-1; i++)
						{
							XDrawLine(theDisplay, BBoard[grp_id].bb_pixmap, 
								BBoard[grp_id].bb_gc, 
								BBoardObj[grp_id].points[2*i], 
								BBoardObj[grp_id].points[2*i+1], 
								BBoardObj[grp_id].points[2*i+2], 
								BBoardObj[grp_id].points[2*i+3]) ;
							XDrawLine(theDisplay, win, BBoard[grp_id].bb_bgc, 
								BBoardObj[grp_id].points[2*i], 
								BBoardObj[grp_id].points[2*i+1], 
								BBoardObj[grp_id].points[2*i+2], 
								BBoardObj[grp_id].points[2*i+3]) ;
						}

						XDrawLine(theDisplay, BBoard[grp_id].bb_pixmap, 
								BBoard[grp_id].bb_gc,
								BBoardObj[grp_id].points[2*i], 
								BBoardObj[grp_id].points[2*i+1], 
								BBoardObj[grp_id].points[0], 
								BBoardObj[grp_id].points[1]) ;
						XDrawLine(theDisplay, win, BBoard[grp_id].bb_bgc,
								BBoardObj[grp_id].points[2*i], 
								BBoardObj[grp_id].points[2*i+1], 
								BBoardObj[grp_id].points[0], 
								BBoardObj[grp_id].points[1]) ;
XSetForeground(theDisplay, BBoard[grp_id].bb_gc, Black) ;
XSetForeground(theDisplay, BBoard[grp_id].bb_bgc, Black) ;
						break ;

					case NONE:
					default :
						break ;
				}/*end of switch*/
				XSetFunction( theDisplay, BBoard[grp_id].bb_gc, GXxor);
				XSetFunction( theDisplay, BBoard[grp_id].bb_bgc, GXxor);
				DrawingOn[grp_id] = FALSE ;
				/*Send to the group*/
				if (( CurrBBoardObj[grp_id] != TEXT) ||
					( CurrBBoardObj[grp_id] != NONE ) )
					T_send_BBoardObj(grp_id, CurrBBoardObj[grp_id],
						Line_Style[grp_id], Fill_Style[grp_id],
						Line_Width[grp_id],
						Curr_Color[grp_id] ,
						BBoardObj[grp_id].points,
						BBoardObj[grp_id].current_point) ;

				/* Terminate the polyline */
				DrawingOn[grp_id] = FALSE ;
				xv_set(BBoard[grp_id].infotxt,
					PANEL_VALUE, "Select one of the above objects",
					PANEL_VALUE_DISPLAY_LENGTH, 32, NULL) ;
				BBoardObj[grp_id].current_point = 0 ;
				XSetLineAttributes( theDisplay, BBoard[grp_id].bb_gc, 1,
					Line_Style[grp_id], CapProjecting, JoinMiter) ;
				XSetLineAttributes( theDisplay, BBoard[grp_id].bb_bgc, 1,
					Line_Style[grp_id], CapProjecting, JoinMiter) ;

				}/*end of if */
			break ;
			}/* end case */
			case	ACTION_MENU:
			case	MS_RIGHT:
				break ;

			default :
				return ;
		}/*switch*/
	}/*else*/

		
}

/*-------------------------------------------------process_circ()-------------*/

void	process_circ( item, event)
Panel_item	item ;
Event *event ;
{	
	Window	win ;
	int		grp_id ;

	grp_id = find_group_from_bbpanel( event) ;
	
	if (DrawingOn[grp_id] == FALSE)
	{
		win = (Window) xv_get(xv_get( BBoard[grp_id].bb_canvas, 
							CANVAS_NTH_PAINT_WINDOW, 0), XV_XID) ;
		if ( InTextMode[grp_id] == TRUE)
		{
				reset_marker(grp_id) ;
				InTextMode[grp_id] = FALSE ;
		}
		xv_set(BBoard[grp_id].infotxt,
			PANEL_VALUE, "Select centre and draw circle by dragging",
			PANEL_VALUE_DISPLAY_LENGTH, 50, NULL) ;
		xv_set(BBoard[grp_id].object_indicator,
			PANEL_VALUE, "CIRCLE",
			PANEL_VALUE_DISPLAY_LENGTH, 6, NULL) ;

		/*Define the cursor*/
		XDefineCursor(theDisplay, win, circle_cursor) ;
	
		CurrBBoardObj[grp_id] = CIRC ;
		BBoardObj[grp_id].current_point = 0 ;
	}
}

/*---------------------------------------------process_polyline()-------------*/
void	process_polyline( item, event)
Panel_item	item ;
Event *event ;
{
	Window	win ;
	int	grp_id;

	grp_id = find_group_from_bbpanel( event) ;
	
	if (DrawingOn[grp_id] == FALSE)
	{
		win = (Window) xv_get(xv_get( BBoard[grp_id].bb_canvas, 
							CANVAS_NTH_PAINT_WINDOW, 0), XV_XID) ;
		if ( InTextMode[grp_id] == TRUE)
		{
				reset_marker(grp_id) ;
				InTextMode[grp_id] = FALSE ;
		}
		xv_set(BBoard[grp_id].infotxt,
			PANEL_VALUE, "Draw multiple lines by clicking points. End by middle button click",
			PANEL_VALUE_DISPLAY_LENGTH, 80, NULL) ;
		xv_set(BBoard[grp_id].object_indicator,
			PANEL_VALUE, "POLYLINE",
			PANEL_VALUE_DISPLAY_LENGTH, 8, NULL) ;
		
		XUndefineCursor(theDisplay, win) ;
		CurrBBoardObj[grp_id] = POLYLINE ;
		BBoardObj[grp_id].current_point = 0 ;
	}
}

/*----------------------------------------------process_polygon()-------------*/
void	process_polygon(item, event)
Panel_item	item ;
Event	*event ;
{
	Window	win ;
	int	grp_id;

	grp_id = find_group_from_bbpanel( event) ;
	
	if ( DrawingOn[grp_id] == FALSE )
	{
		win = (Window) xv_get(xv_get( BBoard[grp_id].bb_canvas, 
							CANVAS_NTH_PAINT_WINDOW, 0), XV_XID) ;
		if ( InTextMode[grp_id] == TRUE)
		{
				reset_marker(grp_id) ;
				InTextMode[grp_id] = FALSE ;
		}
		xv_set(BBoard[grp_id].infotxt,
			PANEL_VALUE, "Draw a polygon by clicking vertices. ",
			PANEL_VALUE_DISPLAY_LENGTH, 40, NULL) ;
		xv_set(BBoard[grp_id].object_indicator,
			PANEL_VALUE, "POLYGON",
			PANEL_VALUE_DISPLAY_LENGTH, 7, NULL) ;
		XUndefineCursor(theDisplay, win) ;

		CurrBBoardObj[grp_id] = POLYGON ;
		BBoardObj[grp_id].current_point = 0 ;
	}
}

/*---------------------------------------------process_erase()-------------*/
void	process_erase( item, event)
Panel_item	item ;
Event	*event;
{
	Window	win ;
	int	grp_id;

	grp_id = find_group_from_bbpanel( event) ;
	
	if ( DrawingOn[grp_id] == FALSE)
	{
		win = (Window) xv_get(xv_get( BBoard[grp_id].bb_canvas, 
							CANVAS_NTH_PAINT_WINDOW, 0), XV_XID) ;
		if ( InTextMode[grp_id] == TRUE)
		{
				reset_marker(grp_id) ;
				InTextMode[grp_id] = FALSE ;
		}
		xv_set(BBoard[grp_id].infotxt,
			PANEL_VALUE, "Erase a rectangular area",
			PANEL_VALUE_DISPLAY_LENGTH, 80, NULL) ;
		xv_set(BBoard[grp_id].object_indicator,
			PANEL_VALUE, "ERASE",
			PANEL_VALUE_DISPLAY_LENGTH, 5, NULL) ;
		XUndefineCursor(theDisplay, win) ;

		CurrBBoardObj[grp_id] = ERASE ;
		BBoardObj[grp_id].current_point = 0 ;
	}
}

/*-------------------------------------------------process_rect()-------------*/
void	process_rect( item, event)
Panel_item	item ;
Event *event ;
{
	Window	win ;
	int	grp_id;

	grp_id = find_group_from_bbpanel( event) ;
	

	if ( DrawingOn[grp_id] == FALSE)
	{
		win = (Window) xv_get(xv_get( BBoard[grp_id].bb_canvas, 
							CANVAS_NTH_PAINT_WINDOW, 0), XV_XID) ;
		if ( InTextMode[grp_id] == TRUE)
		{
				reset_marker(grp_id) ;
				InTextMode[grp_id] = FALSE ;
		}
		xv_set(BBoard[grp_id].infotxt,
			PANEL_VALUE, "Draw a rectangle by clicking vertices",
			PANEL_VALUE_DISPLAY_LENGTH, 80, NULL) ;
		xv_set(BBoard[grp_id].object_indicator,
			PANEL_VALUE, "RECTANGLE",
			PANEL_VALUE_DISPLAY_LENGTH, 9, NULL) ;

		/*Set the cursor*/
		XDefineCursor(theDisplay, win, top_left) ;

		CurrBBoardObj[grp_id] = RECT ;
		BBoardObj[grp_id].current_point = 0 ;
	}
}

/*---------------------------------------------process_arc()-------------*/
void	process_arc( item, event)
Panel_item	item ;
Event *event ;
{
	Window	win ;
	int	grp_id;

	grp_id = find_group_from_bbpanel( event) ;
	
	if ( DrawingOn[grp_id] == FALSE)
	{
		win = (Window) xv_get(xv_get( BBoard[grp_id].bb_canvas, 
							CANVAS_NTH_PAINT_WINDOW, 0), XV_XID) ;
		if ( InTextMode[grp_id] == TRUE)
		{
				reset_marker(grp_id) ;
				InTextMode[grp_id] = FALSE ;
		}
		xv_set( BBoard[grp_id].infotxt, PANEL_VALUE,
			"Select three points to draw an arc", 
			PANEL_VALUE_DISPLAY_LENGTH, 50, NULL) ;
	
		xv_set(BBoard[grp_id].object_indicator,
			PANEL_VALUE, "ARC",
			PANEL_VALUE_DISPLAY_LENGTH, 3, NULL) ;

		XUndefineCursor(theDisplay, win) ;
		CurrBBoardObj[grp_id] = ARC ;
		BBoardObj[grp_id].current_point = 0 ;
	}
}

/*-------------------------------------------------process_grid()-------------*/
void	process_grid( item, event)
Panel_item	item ;
Event	*event ;
{

	int	grp_id;

	grp_id = find_group_from_bbpanel( event) ;
	
	if ( GRID[grp_id] == ON)
		GRID[grp_id] = OFF ;
	else
		GRID[grp_id] = ON ;

	draw_grid(grp_id) ;
}

/*----------------------------------------------------draw_grid()-------------*/
void	draw_grid( grp_id)
int	grp_id;
{
	Window	win ;
	int		x,y ;


	win = (Window) xv_get(xv_get( BBoard[grp_id].bb_canvas, 
							CANVAS_NTH_PAINT_WINDOW, 0),
			XV_XID) ;

	/* Set the linestyle first*/
	XSetFillStyle( theDisplay, BBoard[grp_id].bb_bgc, LineSolid) ;
	XSetLineAttributes(theDisplay, BBoard[grp_id].bb_bgc, 1, LineOnOffDash,
		CapProjecting, JoinMiter) ;

	if (GRID[grp_id] == OFF)
		XSetFunction(theDisplay, BBoard[grp_id].bb_bgc, GXcopyInverted) ;
	else
		XSetFunction(theDisplay, BBoard[grp_id].bb_bgc, GXcopy) ;
		

	/* Draw the grid ... Is this efficient? */
	for ( x=0; x< BBFRM_WIDTH; x+=GRID_STEP)
	{
		XDrawLine( theDisplay, BBoard[grp_id].bb_pixmap, BBoard[grp_id].bb_bgc,
			x, 0, x, BBFRM_HEIGHT) ;
	}

	for ( y=0; y<BBFRM_HEIGHT; y+=GRID_STEP)
	{
		XDrawLine( theDisplay, BBoard[grp_id].bb_pixmap, BBoard[grp_id].bb_bgc,
			0, y, BBFRM_WIDTH, y) ;
	}

}

				

	
/*----------------------------------------------------draw_bb()---------------*/

void	draw_bb(canvas, pw, dpy, xwin, xrects)
Canvas	canvas ;
Xv_Window	pw;
Display		*dpy;
Window	xwin ;
Xv_xrectlist	*xrects ;
{
	int		i;
/****WORKHERE: Probs on color
	for ( i=0; i<MAX_GRP_NO; i++)
	{
		if ( BBoard[i].bb_canvas == canvas)
			XCopyArea( theDisplay, BBoard[i].bb_pixmap, xwin, BBoard[i].bb_gc, 
				0,0, BBFRM_WIDTH, BBFRM_HEIGHT, 0,0) ;
	}
******/
}


/*-------------------------------------------------calculate_obj_size()-------*/
void	calculate_obj_size( grp_id, cx, cy, pvx, pvy, px, py, b, h)
int	grp_id ;
short	cx, cy, pvx,pvy ;
short	*px, *py, *b, *h ;
{

	(*b) = cx-pvx;
	if ((*b) < 0)
	{	
		(*b) = -(*b) ;
		(*px) = cx;
	}
	else
		(*px) = pvx;
	(*h) = cy-pvy ;
	if ((*h) < 0)
	{
		(*h) = -(*h) ;
		(*py) = cy ;
	}
	else
		(*py) = pvy  ;
}


/*--------------------------------------------------file_menu_proc()----------*/
void	file_menu_proc( menu, menu_item)
Menu 	menu ;
Menu_item	menu_item ;
{
}

/*--------------------------------------------------color_proc()--------------*/
void	color_proc( menu, menu_item)
Menu 	menu ;
Menu_item	menu_item ;
{
	Window	win ;
	int	grp_id ;

	
	XColor	rgb_col, hw_col ;
	int		status ;

	grp_id = find_group_from_bbmenu(menu) ;

/*WORKHERE: find a better way to do this */

	if (!(strcmp((char *)xv_get(menu_item, MENU_STRING),"Black")))
			Curr_Color[grp_id] =  0;
	else
	if (!(strcmp((char *)xv_get(menu_item, MENU_STRING),"Blue")))
			Curr_Color[grp_id] = 1 ;
	else
	if (!(strcmp((char *)xv_get(menu_item, MENU_STRING),"Brown")))
			Curr_Color[grp_id] = 2 ;
	else
	if (!(strcmp((char *)xv_get(menu_item, MENU_STRING),"Cyan")))
			Curr_Color[grp_id] = 3 ;
	else
	if (!(strcmp((char *)xv_get(menu_item, MENU_STRING),"Green")))
			Curr_Color[grp_id] = 4 ;
	else
	if (!(strcmp((char *)xv_get(menu_item, MENU_STRING),"Khaki")))
			Curr_Color[grp_id] = 5 ;
	else
	if (!(strcmp((char *)xv_get(menu_item, MENU_STRING),"Orange")))
			Curr_Color[grp_id] = 6 ;
	else
	if (!(strcmp((char *)xv_get(menu_item, MENU_STRING),"Red")))
			Curr_Color[grp_id] = 7 ;
	else
	if (!(strcmp((char *)xv_get(menu_item, MENU_STRING),"Violet")))
			Curr_Color[grp_id] = 8 ;
	else
	if (!(strcmp((char *)xv_get(menu_item, MENU_STRING),"Yellow")))
			Curr_Color[grp_id] = 9 ;
	else
		Curr_Color[grp_id] = 0 ;

/* Allocate the color here */

	if (disp_depth > 1)  
	{
		status = XLookupColor(theDisplay, theCmap, 
					(char *)xv_get(menu_item, MENU_STRING), &rgb_col, &hw_col) ;
		if (status != 0)
		{
			status = XAllocColor( theDisplay, theCmap, &hw_col) ;
			if (status !=0)
				col_pixel[Curr_Color[grp_id]] = hw_col.pixel ;
		}
	}
	else 
	{
		if (strcmp("White",(char *)xv_get(menu_item, MENU_STRING)))
			col_pixel[Curr_Color[grp_id]] = (unsigned long)Black ;
		else
			col_pixel[Curr_Color[grp_id]] = (unsigned long)White ;
	}
				
/***	
	XSetForeground( theDisplay, BBoard[grp_id].bb_gc, col_pixel[Curr_Color[grp_id]]) ;
	XSetBackground( theDisplay, BBoard[grp_id].bb_gc, col_pixel[9]) ;
	XSetForeground( theDisplay, BBoard[grp_id].bb_bgc, col_pixel[Curr_Color[grp_id]]) ;
	XSetBackground( theDisplay, BBoard[grp_id].bb_bgc, col_pixel[9]) ;
***/
printf("Setting Color of grp %d to %ld \n", grp_id, col_pixel[Curr_Color[grp_id]]) ;

}

/*-------------------------------------------------fill_style_proc()----------*/
void	fill_style_proc( menu, menu_item)
Menu 	menu ;
Menu_item	menu_item ;
{
	int	grp_id ;
	Window	xwin ;
	Server_image	*img ;
	int		 i ;

	grp_id = find_group_from_bbmenu(menu) ;

	img = (Server_image *)xv_get(menu_item, MENU_IMAGE)  ;
	for ( i=0; i < NUMBER_OF_PATTERNS; i++)
	{
		if ( img == fimg[i])
		{
			Fill_Style[grp_id] = i ;
			break ;
		}
	}
	
	
	xwin = xv_get( canvas_paint_window( BBoard[grp_id].bb_canvas),
					 XV_XID) ;

	if (Fill_Style[grp_id] != NONE)
		fill_tile[grp_id] = XCreatePixmapFromBitmapData( theDisplay, xwin,
					fill_images[Fill_Style[grp_id]], 8, 8, 1, 0, 1) ;

}

/*-------------------------------------------------line_width_proc()----------*/
void	line_width_proc( menu, menu_item)
Menu 	menu ;
Menu_item	menu_item ;
{
	int	grp_id ;
	Window	xwin ;

	grp_id = find_group_from_bbmenu(menu) ;
	
	if (!(strcmp((char *)xv_get(menu_item, MENU_STRING),"1")))
			Line_Width[grp_id] = 1 ;
	else
	if (!(strcmp((char *)xv_get(menu_item, MENU_STRING),"2")))
			Line_Width[grp_id] = 2 ;
	else
	if (!(strcmp((char *)xv_get(menu_item, MENU_STRING),"3")))
			Line_Width[grp_id] = 3 ;
	else
	if (!(strcmp((char *)xv_get(menu_item, MENU_STRING),"4")))
			Line_Width[grp_id] = 4 ;
	else
	if (!(strcmp((char *)xv_get(menu_item, MENU_STRING),"5")))
			Line_Width[grp_id] = 5 ;

}

/*-------------------------------------------------line_style_proc()----------*/
void	line_style_proc( menu, menu_item)
Menu 	menu ;
Menu_item	menu_item ;
{
	int	grp_id ;
	Window	xwin ;

	grp_id = find_group_from_bbmenu(menu) ;

	if (!(strcmp((char *)xv_get(menu_item, MENU_STRING),"Solid")))
			Line_Style[grp_id] = LineSolid ;
	else
	if (!(strcmp((char *)xv_get(menu_item, MENU_STRING),"OnOffDash")))
			Line_Style[grp_id] = LineOnOffDash ;
	else
	if (!(strcmp((char *)xv_get(menu_item, MENU_STRING),"DoubleDash")))
			Line_Style[grp_id] = LineSolid ;

	xwin = xv_get( canvas_paint_window( BBoard[grp_id].bb_canvas),
					 XV_XID) ;

	XSetLineAttributes( theDisplay, BBoard[grp_id].bb_gc, Line_Width[grp_id],
		Line_Style[grp_id], CapProjecting, JoinMiter) ;

}


/* routines for fonts */

/*-------------------------------------------------font_family_proc()---------*/
void	font_family_proc( menu, menu_item)
Menu 	menu ;
Menu_item	menu_item ;
{
	int	grp_id ;
	Window	xwin ;
	Xv_font	font ;

	grp_id = find_group_from_bbmenu(menu) ;
	
	font = (Xv_Font)xv_find( BBoard[grp_id].bb_frame, FONT,
		FONT_FAMILY,	xv_get(menu_item, MENU_STRING) ,
		FONT_STYLE,		font_styles[Font_Style[grp_id]],
		FONT_SCALE, 	Font_Size[grp_id],
		FONT_SIZES_FOR_SCALE, 12, 14, 16, 22,
		NULL) ;
	
	setfontfamily( grp_id, xv_get(menu_item, MENU_STRING)) ;
	
	if ( font != NULL)
	{	
		/* Free the current definiton of font */
		/*XFreeFont( cur_font[grp_id]) ;*/

		cur_font [grp_id] = (XFontStruct *)xv_get(font, FONT_INFO) ;
		cur_char_width[grp_id] = cur_font[grp_id]->max_bounds.rbearing
									- cur_font[grp_id]->min_bounds.lbearing ;
		cur_char_height[grp_id] = cur_font[grp_id]->max_bounds.ascent 
									+ cur_font[grp_id]->max_bounds.descent ;
		XSetFont( theDisplay, BBoard[grp_id].bb_gc, cur_font[grp_id]->fid) ;
		XSetFont( theDisplay, BBoard[grp_id].bb_bgc, cur_font[grp_id]->fid) ;
	}
}
	
/*-------------------------------------------------font_style_proc()---------*/
void	font_style_proc( menu, menu_item)
Menu 	menu ;
Menu_item	menu_item ;
{
	int	grp_id ;
	Window	xwin ;
	Xv_Font	font ;

	grp_id = find_group_from_bbmenu(menu) ;
	
	font = (Xv_Font)xv_find( BBoard[grp_id].bb_frame, FONT,
		FONT_FAMILY,	font_families[Font_Family[grp_id]],
		FONT_STYLE,		xv_get(menu_item, MENU_STRING),
		FONT_SCALE, 	Font_Size[grp_id],
		FONT_SIZES_FOR_SCALE, 12, 14, 16, 22,
		NULL) ;

	setfontstyle( grp_id, xv_get(menu_item, MENU_STRING)) ;
	
	if ( font != NULL)
	{	
		/* Free the current definiton of font */
		/*XFreeFont( cur_font[grp_id]) ;*/
		cur_font [grp_id]= (XFontStruct *)xv_get(font, FONT_INFO) ;
		cur_char_width[grp_id] = cur_font[grp_id]->max_bounds.rbearing
									- cur_font[grp_id]->min_bounds.lbearing ;
		cur_char_height[grp_id] = cur_font[grp_id]->max_bounds.ascent 
									+ cur_font[grp_id]->max_bounds.descent ;
		XSetFont( theDisplay, BBoard[grp_id].bb_gc, cur_font[grp_id]->fid) ;
		XSetFont( theDisplay, BBoard[grp_id].bb_bgc, cur_font[grp_id]->fid) ;
	}
}

/*---------------------------------------------------font_size_proc()---------*/
void	font_size_proc( menu, menu_item)
Menu	menu;
Menu_item	menu_item;
{
	int	grp_id ;
	Xv_font	font ;
	Panel	panel ;
	int		i ;

	grp_id = find_group_from_bbmenu(menu) ;
	setfontscale( grp_id, xv_get(menu_item, MENU_STRING)) ;

	font = (Xv_Font)xv_find( BBoard[grp_id].bb_frame, FONT,
		FONT_FAMILY,	font_families[Font_Family[grp_id]],
		FONT_STYLE,		font_styles[Font_Style[grp_id] ],
		FONT_SCALE, 	Font_Size[grp_id],
		FONT_SIZES_FOR_SCALE, 12, 14, 16, 22,
		NULL) ;

	
	if (font != NULL)
	{	
		/* Free the current definiton of font */
		/*XFreeFont( cur_font[grp_id]) ;*/
		cur_font[grp_id] = (XFontStruct *)xv_get(font, FONT_INFO) ;
		cur_char_width[grp_id] = cur_font[grp_id]->max_bounds.rbearing
									- cur_font[grp_id]->min_bounds.lbearing ;
		cur_char_height[grp_id] = cur_font[grp_id]->max_bounds.ascent 
									+ cur_font[grp_id]->max_bounds.descent ;
		XSetFont( theDisplay, BBoard[grp_id].bb_gc, cur_font[grp_id]->fid) ;
		XSetFont( theDisplay, BBoard[grp_id].bb_bgc, cur_font[grp_id]->fid) ;
	}
}

/*-------------------------------------------------setfontscale()-------------*/
void	setfontscale (grp_id, str)
int	grp_id ;
char	*str ;
{

	if( strcmp(str, "Small") == 0 )
		Font_Size[grp_id] = WIN_SCALE_SMALL ;
	else if ( strcmp(str,"Medium") == 0)
		Font_Size[grp_id] = WIN_SCALE_MEDIUM ;
	else if ( strcmp(str,"Large") == 0)
		Font_Size[grp_id] = WIN_SCALE_LARGE ;
	else if ( strcmp(str,"X-Large") == 0)
		Font_Size[grp_id] = WIN_SCALE_EXTRALARGE ;

}

/*-------------------------------------------------setfontfamily()------------*/
void	setfontfamily(grp_id, str)
int		grp_id ;
char	*str ;
{
	int		i ;

	Font_Family[grp_id] = -1 ;
	for (i=0 ; i < NUMBER_OF_FONT_FAMILIES; i++)
		if ( strcmp( str, font_families[i]) == 0 )
			Font_Family[grp_id] = i ;
}

/*-------------------------------------------------setfontstyle()------------*/
void	setfontstyle(grp_id, str)
int		grp_id ;
char	*str ;
{

	int		i ;

	Font_Style[grp_id] = -1 ;
	for (i=0; i<NUMBER_OF_FONT_STYLES; i++)
	{
		if ( strcmp( str, font_styles[i]) == 0 )
			Font_Style[grp_id] = i ;
	}
}


/*-------------------------------------------------process_text()------------*/
void	process_text( item, event)
Panel_item	item ;
Event *event ;
{
	int	grp_id;

	grp_id = find_group_from_bbpanel( event) ;
	
	if (DrawingOn[grp_id] == FALSE)
	{
		if ( InTextMode[grp_id] == TRUE)
		{
				reset_marker(grp_id) ;
				InTextMode[grp_id] = FALSE ;
		}
		xv_set( BBoard[grp_id].infotxt, 
			PANEL_VALUE,"Select position to enter text ", 
			PANEL_VALUE_DISPLAY_LENGTH, 50, NULL) ;
	

		xv_set(BBoard[grp_id].object_indicator,
			PANEL_VALUE, "TEXT",
			PANEL_VALUE_DISPLAY_LENGTH, 4, NULL) ;
		CurrBBoardObj[grp_id] = TEXT ;
		BBoardObj[grp_id].current_point = 0 ;
	}
}

/*-------------------------------------------------process_todo()------------*/
void	process_todo( item, event)
Panel_item	item ;
Event *event ;
{

	errmsg("TODO") ;
}

/*-------------------------------------------------display_bb_char()----------*/
display_bb_char( dptr)
CL_PKT	*dptr ;
{
	int	grp_id ;
	Window	xwin ;
	Xv_font	font ;
	XFontStruct	*rcv_font ;
	
	grp_id = (int)dptr->grp_id ;
	xwin = xv_get( canvas_paint_window( BBoard[grp_id].bb_canvas),
					 XV_XID) ;

	 font = (Xv_Font)xv_find( BBoard[grp_id].bb_frame, FONT,
        FONT_FAMILY,    font_families[dptr->font_family] ,
        FONT_STYLE,     font_styles[dptr->font_style],
        FONT_SCALE,     dptr->font_size,
        FONT_SIZES_FOR_SCALE, 12, 14, 16, 22,
        NULL) ;

	if ( font != NULL)
	{
		rcv_font = (XFontStruct *)xv_get(font, FONT_INFO) ;
		rcv_char_width[grp_id] = cur_font[grp_id]->max_bounds.rbearing
									- cur_font[grp_id]->min_bounds.lbearing ;
		XSetFont( theDisplay, BBoard[grp_id].bb_gc, rcv_font->fid) ;
		XSetFont( theDisplay, BBoard[grp_id].bb_bgc, rcv_font->fid) ;
	}

	if ((dptr->letter == BACKSPACE) || ( dptr->letter== DELETE) )
	{
				XSetFillStyle( theDisplay, BBoard[grp_id].bb_gc, FillSolid) ;
				XSetFillStyle( theDisplay, BBoard[grp_id].bb_bgc, FillSolid) ;
				XSetFunction( theDisplay, BBoard[grp_id].bb_bgc, GXcopy) ;
				XSetForeground( theDisplay, BBoard[grp_id].bb_bgc, col_pixel[Curr_Color[grp_id]]) ;
				XSetBackground( theDisplay, BBoard[grp_id].bb_bgc, Black) ;
				XSetFunction( theDisplay, BBoard[grp_id].bb_gc, GXcopy) ;
				XSetForeground( theDisplay, BBoard[grp_id].bb_gc, col_pixel[Curr_Color[grp_id]]) ;
				XSetBackground( theDisplay, BBoard[grp_id].bb_gc, Black) ;
				XFillRectangle(theDisplay, xwin, BBoard[grp_id].bb_bgc, 
					dptr->pos_x-rcv_font->max_bounds.lbearing
					-rcv_font->max_bounds.rbearing-rcv_font->min_bounds.lbearing,
					dptr->pos_y -rcv_font->max_bounds.ascent,
				rcv_font->min_bounds.lbearing+rcv_font->max_bounds.rbearing,
				rcv_font->max_bounds.ascent+rcv_font->max_bounds.descent) ;

				XFillRectangle(theDisplay, BBoard[grp_id].bb_pixmap, 
					BBoard[grp_id].bb_gc,
					dptr->pos_x-rcv_font->max_bounds.lbearing
					-rcv_font->max_bounds.rbearing-rcv_font->min_bounds.lbearing,
					dptr->pos_y -rcv_font->max_bounds.ascent,
				rcv_font->min_bounds.lbearing+rcv_font->max_bounds.rbearing,
				rcv_font->max_bounds.ascent+rcv_font->max_bounds.descent) ;
				XSetForeground( theDisplay, BBoard[grp_id].bb_gc, Black) ;
				XSetBackground( theDisplay, BBoard[grp_id].bb_gc, col_pixel[Curr_Color[grp_id]]) ;
				XSetForeground( theDisplay, BBoard[grp_id].bb_bgc, Black) ;
				XSetBackground( theDisplay, BBoard[grp_id].bb_bgc, col_pixel[Curr_Color[grp_id]]) ;
				XSetFunction( theDisplay, BBoard[grp_id].bb_bgc, GXxor) ;
				XSetFunction( theDisplay, BBoard[grp_id].bb_gc, GXxor) ;
	}
	else
	{
		XDrawString( theDisplay, BBoard[grp_id].bb_pixmap, BBoard[grp_id].bb_gc, 
			dptr->pos_x, dptr->pos_y, &(dptr->letter), 1) ;
		XDrawString( theDisplay, xwin, BBoard[grp_id].bb_bgc, dptr->pos_x, 
			dptr->pos_y, &(dptr->letter), 1) ;
	}


	/*XFreeFont(rcv_font) ;*/

	/*Set to orig font */
	XSetFont( theDisplay, BBoard[grp_id].bb_gc, cur_font[grp_id]->fid) ;
	XSetFont( theDisplay, BBoard[grp_id].bb_bgc, cur_font[grp_id]->fid) ;
}

/*-------------------------------------------------fill_polypoints()----------*/
fill_polypoints( xpts, grp_id) 
XPoint	*xpts ;
int		grp_id ;
{
	
	int	 j;
	
	for ( j=0; j< BBoardObj[grp_id].current_point; j++ )
	{
		/* Copy the point into xpts */
		xpts[j].x = BBoardObj[grp_id].points[2*j] ;
		xpts[j].y = BBoardObj[grp_id].points[2*j+1] ;
	}

}

/*------------------------------------------------fill_rpolypoints()----------*/
fill_rpolypoints( xpts, points, npoints) 
XPoint	*xpts ;
short	*points ;
int		npoints ;
{
	
	int	 j;
	
	for ( j=0; j< npoints; j++ )
	{
		/* Copy the point into xpts */
		xpts[j].x = points[2*j] ;
		xpts[j].y = points[2*j+1] ;
	}

}


/*----------------------------------------------------reset_marker()----------*/
reset_marker ( grp_id)
int		grp_id ;
{

	Window	win ;

	win = xv_get( canvas_paint_window( BBoard[grp_id].bb_canvas),
					 XV_XID) ;
	XSetFillStyle( theDisplay, BBoard[grp_id].bb_gc, FillSolid) ;
	XSetFillStyle( theDisplay, BBoard[grp_id].bb_bgc, FillSolid) ;
	XSetLineAttributes(theDisplay, BBoard[grp_id].bb_gc, 1, LineSolid,
                        CapProjecting, JoinMiter) ;
	XSetLineAttributes(theDisplay, BBoard[grp_id].bb_bgc, 1, LineSolid,
                        CapProjecting, JoinMiter) ;

	XSetFunction(theDisplay, BBoard[grp_id].bb_gc, GXxor) ;
	XSetFunction(theDisplay, BBoard[grp_id].bb_bgc, GXxor) ;

	XDrawLine( theDisplay, BBoard[grp_id].bb_pixmap, BBoard[grp_id].bb_gc,
		BBoardObj[grp_id].current_x, 
		BBoardObj[grp_id].current_y-MARK_HEIGHT,
		BBoardObj[grp_id].current_x,
		BBoardObj[grp_id].current_y ) ;
	XDrawLine( theDisplay, win, BBoard[grp_id].bb_bgc,
		BBoardObj[grp_id].current_x, 
		BBoardObj[grp_id].current_y-MARK_HEIGHT,
		BBoardObj[grp_id].current_x,
		BBoardObj[grp_id].current_y ) ;
}


/*----------------------------------------------------process_filesave()------*/
void	process_filesave( item, event)
Panel_item	item ;
Event *event ;
{
	/* Write the pixmap into bitmap file */
	XWriteBitmapFile( theDisplay, "/tmp/bb.bm", BBoard[0].bb_pixmap, BBFRM_WIDTH,
			BBFRM_HEIGHT, -1, -1) ;
	
}	

void	bb_quit()
{
printf("In bb_quit \n") ;
}
/*----------------------------------------------------End of blackboard.c-----*/
