/*----------------------------------------------------------File: io.c--------*/
/******************************************************************************/
/*                                                                            */
/*                                                                            */
/*                              Vartalaap                                     */
/*----------------------------------------------------------------------------*/
/*                  Multimedia Multiparty  Conferencing System                */
/*                                                                            */
/*                                                                            */
/*                  Written by:                                               */
/*                            Yezdezard Lashkari                              */
/*                            Viswanath Ramachandran                          */
/*                            Sanjay Malpani                                  */
/*                             VIJAY TALATI                                   */
/*                             ERNet Lab,                                     */
/*                             Department of Computer Science,                */
/*                             Indian Institute of Technology, Bombay 400076. */
/*                             E-mail : vijay@gateway.iitb.ernet.in           */
/*                                                                            */
/******************************************************************************/

/* 
 *	Common read/write routines for sockets
 */

#include	"inc.h"

/*----------------------------------------------------_WaitForReadable()------*/
/*
 *	_WaitForReadable -- wait on the connection till enough bytes come
 */

_WaitForReadable(conn)
FD	conn;
{
	fd_set	r_mask;
	int	result;

	/*ENTER_PROC("_WaitForReadable");*/

	FD_ZERO(&r_mask);	/* Clear mask */

	do{
		FD_SET(conn, &r_mask);
		result = select(conn +1, &r_mask, NULL, NULL, NULL);

		if(result == -1 && errno != EINTR )
		{
			perror("Error in select");
			exit(1);
		}

	 }while( result <= 0);

}

/*----------------------------------------------------------------Read()------*/
/*
 *	_Read - Read bytes from the connection taking into account incomplete
 *	reads. 
 */

Status	_Read(conn, data, size)
register FD	conn;
register char 	*data;
register long	size;
{
	register long	bytes_read;

	/*ENTER_PROC("_Read");*/

	if(size == 0) return;
	errno= 0;

	while((bytes_read = read(conn, data, (int)size)) != size)
	{
		if(bytes_read > 0)
		{
			size -= bytes_read;
			data += bytes_read;
		}

#ifdef 	EWOULDBLOCK
		else if(errno == EWOULDBLOCK )
			{
				_WaitForReadable(conn);
				errno = 0;
			}
#endif	/* EWOULDBLOCK */

		else if(bytes_read == 0)	/* read failed due to EOF */
			{
				errno = EPIPE;
				ERROR("Read failed due to EOF");
				return(FAILURE);
			}
		else	/* read retd < 0 ; presumably -1 */
		{
			if(errno != EINTR)
				perror("Read failed");

			return(FAILURE);

		}
	}	/* End of while */

	/*Added for SCO Unix which has l ByteOrder */

	return(SUCCESS);

}


/*--------------------------------------------------------------_Write()------*/
/* 
 *	_Write -- Analogous to _Read. Puts bytes on the connection
 */

Status	_Write(conn, size, buf)
register FD	conn;
register long	size;
register char 	*buf;
{
	int n;
	long bytes_left = size;

	unsigned char *p = (unsigned char *)buf;

	DEBUG_LEVEL3(("Writing to FD:%d\n",conn));

	while(bytes_left > 0)
	{
		if(( n = write(conn, (char *)p, (int)bytes_left)) > 0)
		{
			bytes_left -= n;
			p += n;
		}
		else
		{
			DEBUG_LEVEL2(( "Write failed:\nwrite retd:%d\nerrno:%d\n",
					n, errno));

			SYSFATAL("Error in write");
			return(FAILURE);
		 }

	 }	/* end of while */


	return(SUCCESS);

}

/*------------------------------------------------------_WriteToClient()------*/
/*
 *	WriteToClient-- Currently simply puts bytes on the connection
 *	To be reworked later to provide buffering 
 */

Status	WriteToClient(fd, n, buf)
FD	fd;
unsigned int n;
char *	buf;
{
	Status  st;


	st = _Write(fd, n, buf);

	return(st);
}

/*------------------------------------------------------_WriteToServer()------*/
/*
 *	WriteToServer-- Currently simply puts bytes on the connection
 *	To be reworked later to provide buffering 
 */

Status	WriteToServer(fd, n, buf)
FD	fd;
unsigned int n;
char *	buf;
{
	Status  st;


	DEBUG_LEVEL3(("Writing to FD:%d\n",fd));

	st = _Write(fd, n, buf);

	return(st);
}

/*----------------------------------------------------------ReadBuffer()------*/
/*
 *	ReadBuffer-- Currently simply reads bytes from the connection.
 *	To be reworked later to provide buffering 
 */

Status	ReadBuffer(fd, buf, n)
FD	fd;
char *	buf;
unsigned int n;
{
	Status  st;

	/*ENTER_PROC("ReadBuffer");*/

	st = _Read(fd, buf, n);

	return(st);

}

