/*------------------------------------------------------File: protodefs.h-----*/
/******************************************************************************/
/*                                                                            */
/*                                                                            */
/*                              Vartalaap                                     */
/*----------------------------------------------------------------------------*/
/*                  Multimedia Multiparty  Conferencing System                */
/*                                                                            */
/*                                                                            */
/*                  Written by:                                               */
/*                            Yezdezard Lashkari                              */
/*                            Viswanath Ramachandran                          */
/*                            Sanjay Malpani                                  */
/*                             VIJAY TALATI                                   */
/*                             ERNet Lab,                                     */
/*                             Department of Computer Science,                */
/*                             Indian Institute of Technology, Bombay 400076. */
/*                             E-mail : vijay@gateway.iitb.ernet.in           */
/*                                                                            */
/******************************************************************************/

/*
 *	This file stores the typedefs for the byte stream protocol
 */

/*
 *	All packets are multiples of 4 byte units --- padded if necessary
 */


#include <stdio.h>

/*	For bit vectors 	*/

#ifdef	WORD64			/* If 64 bit architecture then bit fields needed*/

#define		B16	:16
#define		B32	:32

#else

#define		B16	
#define		B32

#endif			/* WORD64 */

/*	Machine dependent types
 */


typedef		unsigned long	CARD32;
typedef		unsigned short	CARD16;
typedef		unsigned char	CARD8;
typedef		unsigned char	BOOL;
typedef		unsigned char	BYTE;

typedef		CARD8		ID;


/*	
 *	Startup Protocol packets
 */

/* 	Client Library to server	*/

typedef	struct{

	CARD8	byteOrder;				/* 'l' or 'B' */
	BYTE	pad1;
	CARD32	pid	B32;				/* User's pid	*/
	CARD16	nBytesAuthProto	B16;	/* No of bytes in auth protocol*/
	CARD16	nBytesAuthString B16;	/* No of bytes in auth string  */
	CARD16 	nBytesUserName	B16;	/* No of bytes in user name string*/
	CARD32	nBytesUserPhoto B32 ;   /* No of bytes in users photo */

		}LL_ConnClientPrefix;


/*	Common to servers		*/

typedef	struct{

	BOOL	success;				/* Did authorisation succeed ? */
	BYTE	lengthReason;			/* If failed -- reason 	       */
	CARD16	pad	B16;

		}ConnSetupPrefix;


/*	
 *	Requests -- All requests are of  12 bytes , padded if necessary 
 */

/* Generic Request structure */

typedef	struct{

	CARD8	reqType;				/* Opcode of request 		*/
	CARD8	data0;					/* Request specific fields 	*/
	CARD16	data1	B16;			
	CARD32	data2	B32;
	CARD32	data3	B32;

		}T_Req;


/*	Client Library to Local Server	*/

typedef	struct{

	CARD8	reqType;			/* _MakeGroupReq*/
	BYTE	pad;
	CARD16	pad1	B16;		
	CARD32	pad2	B32;
	CARD32	pad3	B32;

		}LL_MakeGroupReq;

typedef	struct{

	CARD8	reqType;			/* _JoinGroupReq*/
	CARD8	pad1	;
	ID		gid	;				/* Group id (8 bit) to join up to*/
	CARD8	pad2	;
	CARD32	pad3	B32 ;		
	CARD32	pad4	B32;

		}LL_JoinGroupReq;

typedef	struct{

	CARD8	reqType;			/* _LL_SendDataReq*/
	BYTE	image_type;			/* Type of image, for image data*/
	CARD16	MessageNum	B16 ;
	CARD16	nBytes	B16;		/* No of bytes in string */
	CARD16	pad1	B16 ;
	ID		gid	;				/* Send to which group   */
	CARD8	pad2 ;
	CARD16	pkts_left	B16;	/* the pkts left for image data */	

		}LL_SendDataReq;

typedef	struct{

	CARD8	reqType;			/* _BBoardObjReq*/
	BYTE	obj_type;			/* Type of obj*/
	CARD8	lstyle	;			/* Line style */
	CARD8	fstyle	;			/* Fill style */
	CARD16	nBytes	B16;		/* the  numbe rof bytes */
	CARD16	lwidth	B16 ;
	ID		gid	;				/* Send to which group   */
	CARD8	color ;				/* color to display in */
	CARD16	pad2	B16;		

		}LL_BBoardObjReq;

typedef	struct{

	CARD8	reqType;			/* _BBoardCharReq*/
	CARD8	font_size;				
	CARD8	font_family	;			
	CARD8	font_style	;		
	CARD16	pos_x	B16;	
	CARD16	pos_y	B16 ;
	ID		gid	;				/* Send to which group   */
	BYTE	letter ;			/* the char sent */
	CARD8	color	;		
	CARD8	pad1	;		

		}LL_BBoardCharReq;

typedef	struct{

	CARD8	reqType;			/* _LeaveGroupReq*/
	BYTE	pad1;
	ID		gid	;				/* Group id of the group to leave*/
	CARD8	pad2;			
	CARD32	pad3	B32 ;
	CARD32	pad4	B32 ;

		}LL_LeaveGroupReq;

typedef	struct{

	CARD8	reqType;			/* _HistoryReq */
	BYTE	pad1;
	ID		gid	;				/*Group id -- 8 bit */
	CARD8	pad2 ;
	CARD32	pad3	B32;
	CARD32	pad4	B32;

		}LL_HistoryReq;

typedef	struct{

	CARD8	reqType;			/* _UserListReq */
	BYTE	pad1;
	ID		gid	;				/*Group id  8 bit */
	CARD8	pad2	;
	CARD32	pad3	B32;
	CARD32	pad4	B32;

		}LL_UserListReq;

typedef	struct{

	CARD8	reqType;			/* _AnnotateReq */
	ID		gid	;				/* Group id -- 8 bit */
	CARD8	pad1	;			
	CARD8	userid	;			/* the user id */
	CARD16	From	B16;		/* From character */
	CARD16	To		B16;		/* To character */
	CARD32	pad2	B32;			

		}LL_AnnotateReq;

typedef	struct{

	CARD8	reqType;			/* _ImagemarkReq */
	CARD8	gid;				/* Group id -- 8 bit */
	CARD8	ImgIndx	;			/* Number of image to mark*/
	CARD8	type	;			/* Type (arc, rect etc.) of mark*/
	CARD16	startx ;			/* Coordinates for the */
	CARD16	starty ;			/* mark 			   */
	CARD16	endx ;
	CARD16	endy ;

		}LL_ImagemarkReq;



/*
 *	Replies
 */

typedef	struct{

	CARD8	type;				/* _T_Reply, _T_Error, _T_Data */
	CARD8	opcode;				/* What subclass ??         */
	CARD16 	pad1	B16;	
	CARD32	pad2	B32;
	CARD32	pad3	B32;
	CARD32	pad4	B32;
	
		}T_GenericReply;

typedef	struct{

	CARD8	type;			/* _T_Reply */
	CARD8	opcode;			/* _MakeGroupReply */
	CARD16 	pad1	B16;	/* 0 */
	BOOL	success;		/* Whether valid id 	*/
	CARD8	userid;			/* Indicates which client*/
	CARD16 	MsgNum	B16;	/* Serial no of message */
	ID		gid	;			/* Id retd by server 	*/
	CARD8	pad3;
	CARD16	ImgIndx B16;	/* the index of the image */
	CARD32	errCode	B32;	/* Opcode if request failed */

		}MakeGroupReply;

typedef	struct{

	CARD8	type;			/* _T_Reply */
	CARD8	opcode;			/* _SendReply */
	CARD16 	pad1	B16;	
	BOOL	success;		/* Whether valid id 	*/
	CARD8	pad2;			/* Indicates which client/local server*/
	CARD16 	MsgNum	B16;	/* Serial no of message */
	ID		gid	;			/* Id retd by server 	*/
	CARD8	pad3;
	CARD8	ImgIndx ;		/* the index of the image */
	CARD8	oad4;
	CARD32	errCode	B32;	/* Opcode if request failed */

		}SendReply;

typedef	struct{

	CARD8	type;			/* _T_Reply */
	CARD8	opcode;			/* _HistoryReq */
	CARD16 	pad1	B16;	
	BOOL	success;		/* Whether valid id 	*/
	CARD8	pad2;			/* Indicates which client/local server*/
	CARD16 	pad3	B16;	
	ID		gid	;			/* Id retd by server 	*/
	CARD8	pad4;
	CARD8	pad5;
	CARD8	pad6;
	CARD32	errCode	B32;	/* Opcode if request failed */

		}HistoryReqReply ;

typedef	struct{

	CARD8	type;			/* _T_Reply */
	CARD8	opcode;			/* _UserListReply*/
	CARD16 	pad1	B16;	
	BOOL	success;		/* Whether valid id 	*/
	CARD8	pad2;			
	CARD16 	pad3	B16;	
	ID		gid	;			/* Id retd by server 	*/
	CARD8	pad4;
	CARD8	pad5;
	CARD8	pad6;
	CARD32	errCode	B32;	/* Opcode if request failed */

	} UserListReqReply ;


typedef	struct{

	CARD8	type;			/* _T_Reply */
	CARD8	opcode;			/* _JoinGroupReply*/
	CARD16 	pad1	B16;	
	BOOL	success;		
	CARD8	userid;			/*Indicates the clients id */
	CARD16 	MsgNum	B16;	/* Serial no of message */
	CARD32	errCode	B32;	/* Opcode if request failed */
	ID		gid	;			/* Group id to which to join */
	CARD8	no_of_peers;	/* No of other participants in same group*/
	CARD8	ImgIndx ;		/* the index of the image */
	CARD8	pad4;

		}JoinGroupReply;

typedef	struct{

	CARD8	type;			/* _T_Reply */
	CARD8	opcode;			/* _LeaveGroupReply */
	CARD16 	pad1	B16;	
	BOOL	success;		
	CARD8	pad2;			
	CARD16 	pad3	B16;	
	CARD32	errCode	B32;	/* Opcode if request failed */
	CARD32	pad4	B32;

		}LeaveGroupReply;

typedef	struct{

	CARD8	type;			/* _T_Reply */
	CARD8	opcode;			/* _AudioStartReply */
	CARD16 	port	B16;	/* The audio port offset*/	
	BOOL	success;		/* True if succeeded */
	CARD8	gid;			/* Indicates group */
	CARD16 	pad1	B16;	
	CARD32	errCode	B32;	/* Opcode if request failed */
	CARD32	pad2	B32;

		}AudioStartReply;

typedef	struct{

	CARD8	type;			/* _T_Reply */
	CARD8	opcode;			/* _ImageStartReply*/
	CARD16 	port	B16;	/* The image port offset */	
	BOOL	success;		/* True if succeeded */	
	CARD8	gid;			/* Indicates group */
	CARD16 	pad1	B16;	
	CARD32	errCode	B32;	/* Opcode if request failed */
	CARD32	pad2	B32;

		}ImageStartReply;

typedef	struct{

	CARD8	type;			/* _T_Reply */
	CARD8	opcode;			/* _JoinGroupIndication*/
	CARD16 	pad1	B16;	
	BOOL	success;		
	CARD8	userid;			/* The id of the recently joined user*/
	CARD16 	pad3	B16;	
	CARD32	errCode	B32;	/* Opcode if request failed */
	ID		gid	;			/* Group id to which to join */
	CARD8	pad4;
	CARD8	pad5 ;	
	CARD8	pad6;

		}JoinGroupIndication;

/*
 *	Errors
 */

typedef	struct{

	CARD8	type;			/* _T_Error */
	CARD8	opcode;			/* What type ??         */
	CARD16	length	B16;	/* 0 */
	BYTE	data;			/* reply specific */
	CARD8	from;			/* Indicates which client/local server*/
	CARD16 	reqNo	B16;	/* Serial no of request */
	CARD32	pad1	B32;
	CARD32	pad2	B32;

		}T_Error;

/*	
 *	Data packet 
 */

typedef	struct{

	CARD8	type;				/* _T_Data */
	CARD8	image_type;			/* Type of image */
	CARD16	pkts_left	B16;	/* Number of pkts left */ 
	CARD16	MsgNum		B16;	/* The number of message */
	CARD16	nBytesUser	B16;	/* No of bytes in username string */
	ID		gid		;			/* Group id to send to */
	CARD8	userid;
	CARD8	ImgIndx ;			/* the index of the image */
	CARD8	pad2;
	CARD32	nBytesData	B32;    /* No of bytes in data following */

		}T_Data;

typedef	struct{

	CARD8	type;				/* _BB_Data */
	CARD8	obj_type;			
	CARD8	lstyle	;	
	CARD8	fstyle		;	
	CARD16	nBytesUser	B16;	
	ID		gid		;			/* Group id to send to */
	CARD8	color;
	CARD16	lwidth B16;	
	CARD32	nBytesData	B32;    /* No of bytes in data following */
	CARD16	pad2	B16;

		}BB_Data;

typedef	struct{

	CARD8	type;				/* _BB_Char */
	CARD8	font_size;			
	CARD8	font_family	;	
	CARD8	font_style	;	
	CARD8	color	;	
	CARD8	pad1	;	
	ID		gid		;			/* Group id to send to */
	CARD8	letter;
	CARD16	pos_x B16;	
	CARD16	pos_y B16;	
	CARD32	pad2 B32;	

		}BB_Char;


/*The data struct. for passing annotation command */
typedef	struct{

	CARD8	type;				/* _A_Data */
	CARD8	pad1;				/* Unused */
	CARD8	pad2		B16;	/* The  userid*/
	CARD8	userid		;		/* The  userid*/
	CARD16	From		B16;	/* Starting from character number*/
	CARD16	To		B16;		/* No of bytes in username string */
	ID		gid		;			/* Group id to send to */
	CARD8	pad3;
	CARD8	pad4;
	CARD8	pad5;
	CARD32	pad6	B32;		/* No of bytes in data following */

		}A_Data;

/*The data struct. for passing start audio command */

typedef	struct{

	CARD8	type;				/* _AS_Data */
	CARD8	pad1;			
	CARD16	port		B16;	/* The port number to connect to */
	CARD16	pad2		B16;
	CARD16	pad3		B16;	
	ID		gid	;				/* Group id to send to */
	CARD8	pad4;
	CARD8	pad5;
	CARD8	pad6;
	CARD32	pad7	B32;    

		}AS_Data;

/*The data struct. for passing start image command */

typedef	struct{

	CARD8	type;				/* _IS_Data */
	CARD8	pad1;			
	CARD16	port	B16;		/* The port number to connect to */
	CARD16	pad2	B16;
	CARD16	pad3	B16;	
	ID		gid		;			/* Group id to send to */
	CARD8	pad4;
	CARD8	pad5;
	CARD8	pad6;
	CARD32	pad7	B32;    

		}IS_Data;


/*The data struct. for passing image mark command */
typedef	struct{

	CARD8	type;				/* _IM_Data */
	CARD8	gid	;				/* Group id to send to */
	CARD8	ImgIndx ;			/* The image number to annotate */
	CARD8	mark_type;			/* The mark type */
	CARD16	startx ;
	CARD16	starty ;
	CARD16	endx ;
	CARD16	endy ;
	CARD32	pad1	B32;		/* No of bytes in data following */

		}IM_Data;


/*
 *	Opcodes for requests , replies, errors etc
 */

/*	REQUEST OPCODES				*/

#define	_MakeGroupReq		1
#define	_JoinGroupReq		2
#define	_LeaveGroupReq		3
#define	_SendDataReq		4
#define	_HistoryReq			5
#define	_AnnotateReq		6
#define	_ImageDataReq		7
#define	_ImagemarkReq		8
#define	_UserListReq		9
#define	_AudioDataReq		10
#define	_AudioStartReq		11
#define	_AudioStopReq		12
#define	_ImageStartReq		13
#define	_BBoardObjReq		14
#define	_BBoardCharReq		15

/*	REPLY  OPCODES				*/

#define	_T_Reply			1
#define	_T_Error			2
#define	_T_Data				3
#define	_H_Data				4
#define	_A_Data				5
#define	_I_Data				6
#define	_IM_Data			7
#define	_HI_Data			8
#define	_UL_Data			9
#define	_V_Data				10
#define	_AS_Data			11
#define	_IS_Data			12
#define	_BB_Data			13
#define	_BB_Char			14
#define	_NU_Data			15
#define	_LU_Data			16

/*	Opcodes specific to _T_Reply.		*/

#define	_MakeGroupReply		14
#define	_JoinGroupReply		15
#define	_LeaveGroupReply	16
#define	_HistoryReqReply	17
#define	_SendReply			18
#define	_UserListReqReply	19
#define	_AudioStartReply	20
#define	_ImageStartReply	21
#define	_Unused				100

/*	Error codes that could be retd by _T_Reply*/

#define	_NoError			21
#define	_NoSuchGroup		22
#define	_NoSuchHost			25
#define _NoSpaceLeft		26
#define _OtherFailure		27
#define _GroupAlreadyJoined	28
#define _GroupNotJoined		29
#define	_NoSuchProcess		30
#define _NoMessage			31

#define _HistMessage		32
#define _AnnotateMessage	33
#define _ImagemarkMessage	34
#define _HistImageMessage	35
#define _UserListMessage	36
#define _AudioReqRejected	37
#define _ImageReqRejected	38
#define	_JoinGroupIndication		39
#define	_LeaveGroupIndication		40

/*-------------------------------------------------------End of protodefs.h-----*/
