/*----------------------------------------------------send.c------------------*/
/******************************************************************************/
/*                                                                            */
/*                                                                            */
/*                              Vartalaap                                     */
/*----------------------------------------------------------------------------*/
/*                  Multimedia Multiparty  Conferencing System                */
/*                                                                            */
/*                                                                            */
/*                  Written by:VIJAY TALATI                                   */
/*                             ERNet Lab,                                     */
/*                             Department of Computer Science,                */
/*                             Indian Institute of Technology, Bombay 400076. */
/*                             E-mail : vijay@gateway.iitb.ernet.in           */
/*                                                                            */
/******************************************************************************/

/*	to send a message on a multicast group    */

#include	"../common/inc.h"
#include	"../common/protodefs.h"

#define		EXTERN	extern

#include	"clientlib.h"
#include	"globals.h"


/*---------------------------------------------------------T_send()------------*/
Status T_send(gid,message,nbytes)
int		gid;		/* talk group id */
char	*message;
int		nbytes;		/* no of byters in message */
{
	FD fd = L_sock ;
	T_GenericReply	genrep ; 
	SendReply		rep ;

	/* 	First check for dirty business by the client */
	if(INIT_LIB_FLAG == FALSE)
		FATAL("Library structures NOT initialised");

	if(nbytes == 0)
	{
		ERROR("Length of message is ZERO");
		return(_OtherFailure);
	}
	
		
	/* Check that the user had joined this group -- via the library */
	/* Now check whether he's actually joined the grp */

	if(IsMember(gid) != SUCCESS)
	{
		ERROR("TRYING TO SEND TO GROUP NOT JOINED");
		return(_GroupNotJoined);
	}

	(void)LL_SendSendDataReq(fd, gid, message, nbytes);

	/*Get the reply. The local server returns the message number
      Added here but looks non standard, compared to other primitives*/
	if( _Reply(fd,_SendReply, &genrep) != SUCCESS)
	{
		ERROR("Send reply problem") ;
		return(_OtherFailure) ;
	}

	bcopy ( (char *)&genrep, (char *)&rep, sizeof(T_GenericReply) ) ;
	
	return(_NoError);

}


/*----------------------------------------------------T_sendimage()------------*/
Status	T_sendimage(gid,message,nbytes,pkts_left,msgnum, image_type)
int		gid;		/* talk group id */
char	*message;
int		nbytes;		/* no of byters in message */
int		pkts_left ;
int		*msgnum ;
unsigned short	image_type ;
{
	FD fd = isock ;
	T_GenericReply	genrep ; 
	SendReply		rep ;

	/* 	First check for dirty business by the client */
	if(INIT_LIB_FLAG == FALSE)
		FATAL("Library structures NOT initialised");

	if(nbytes == 0)
	{
		ERROR("Length of message is ZERO");
		return(_OtherFailure);
	}
		
	/* Check that the user had joined this group -- via the library */
	/* Now check whether he's actually joined the grp */

	if(IsMember(gid) != SUCCESS)
	{
		ERROR("TRYING TO SEND TO GROUP NOT JOINED");
		return(_GroupNotJoined);
	}

	(void)LL_SendImageDataReq(fd, gid, message, nbytes, pkts_left, image_type);

	/* Increment the image index */
	ImageIndex[gid] ++ ;

	return(_NoError);
}



/*--------------------------------------------------T_startaudiochannel()------*/
Status T_startaudiochannel(gid)
int 	gid;		/* talk group id */
{
	FD fd = L_sock ;
	T_GenericReply	genrep ; 
	AudioStartReply		rep ;

	/* 	First check for dirty business by the client */
	if(INIT_LIB_FLAG == FALSE)
		FATAL("Library structures NOT initialised");

	/* Now check whether he's actually joined the grp */
	if(IsMember(gid) != SUCCESS)
	{
		ERROR("TRYING TO SEND TO GROUP NOT JOINED");
		printf("TRYING TO SEND TO GROUP NOT JOINED\n");
		return(_GroupNotJoined);
	}

	(void)LL_SendAudioStartReq(fd, gid);

	if (_Reply(fd, _AudioStartReply, &genrep) != SUCCESS)
    {
        ERROR("Wrong reply received") ;
        return(_OtherFailure) ;
    }     

    bcopy((char *)&genrep, (char *)&rep, sizeof(T_GenericReply)) ;
	/* If neg. ack from server, dont connect */
	if ( rep.success == FALSE)
	{
		return(_AudioReqRejected) ;
	}
 

    if ( (asock= ConnectToServAudio ( rep.port)) <  0)
    {
        ERROR("Connecting to Server's Audio port " );
        return(_OtherFailure) ;
    }     

	return(_NoError);
}

/*-----------------------------------------------------T_sendaudiodata()-------*/
Status	T_sendaudiodata(gid,message,nbytes)
int		gid;		/* talk group id */
char	*message;
int		nbytes;		/* no of byters in message */
{
	FD fd = asock ;
	T_GenericReply	genrep ; 
	SendReply		rep ;


	/* 	First check for dirty business by the client */
	if(INIT_LIB_FLAG == FALSE)
		FATAL("Library structures NOT initialised");

	if(nbytes == 0)
	  {
		ERROR("Length of message is ZERO");
		return(_OtherFailure);
	  }
		

	/* Check that the user had joined this group -- via the library */
	/* Now check whether he's actually joined the grp */

	if(IsMember(gid) != SUCCESS)
	 {
		ERROR("TRYING TO SEND TO GROUP NOT JOINED");
		return(_GroupNotJoined);
	 }

	(void)LL_SendAudioData(fd, gid, message, nbytes);

	return(_NoError);

}

/*----------------------------------------------------T_stopaudiochannel()-----*/
Status	T_stopaudiochannel( gid)
int		gid ;
{

	if(INIT_LIB_FLAG == FALSE)
        FATAL("Library structures NOT initialised");
 
	/* Check that the user had joined this group -- via the library */
    /* Now check whether he's actually joined the grp */
 
    if(IsMember(gid) != SUCCESS)
     {
        ERROR("TRYING TO SEND TO GROUP NOT JOINED");
        return(_GroupNotJoined);
     }

	(void)LL_SendAudioStopReq ( asock, gid) ;

	/* No need for a reply here. Safely assume that the request is
		honoured */

	return(_NoError) ;
}

/*-------------------------------------------------------T_sendimagereq()-----*/
Status	T_sendimagereq( gid)
int		gid ;
{
	FD	fd = L_sock ;
	T_GenericReply	genrep ;
	ImageStartReply	rep ;

    /*  First check for dirty business by the client */
    if(INIT_LIB_FLAG == FALSE)
        FATAL("Library structures NOT initialised");

    /* Now check whether he's actually joined the grp */
    if(IsMember(gid) != SUCCESS)
    {
        ERROR("TRYING TO SEND TO GROUP NOT JOINED");
        printf("TRYING TO SEND TO GROUP NOT JOINED\n");
        return(_GroupNotJoined);
    }

	(void)LL_SendImageStartReq(fd, gid) ;

	if (_Reply(fd, _ImageStartReply, &genrep) != SUCCESS)
	{
		ERROR("Wrong reply received");
		return(_OtherFailure) ;
	}

	bcopy((char *)&genrep, (char *)&rep, sizeof(T_GenericReply)) ;

	if ( rep.success == FALSE)
	{
		return(_ImageReqRejected) ;
	}

	if (( isock = ConnectToServImage( rep.port)) < 0)
	{
		ERROR("Connecting to Server's Image port") ;
		return(_OtherFailure) ;
	}
	
	return(_NoError) ;
}


/*-----------------------------------------------------T_send_BBoardObj()-----*/
Status	T_send_BBoardObj(gid,type, lstyle, fstyle, lwidth, color, points,npoints)
int		gid;		/* talk group id */
int		lstyle, lwidth, fstyle ;
int		color ;
int		type ;
short	*points;
int		npoints;		/* no of points in array */
{
	FD fd = L_sock ;
	int			nbytes ;
	char	*message ;
	int		i ;

	/* 	First check for dirty business by the client */
	if(INIT_LIB_FLAG == FALSE)
		FATAL("Library structures NOT initialised");

	if(npoints == 0)
	{
		ERROR("Length of message is ZERO");
		return(_OtherFailure);
	}

	/* Check that the user had joined this group -- via the library */
	/* Now check whether he's actually joined the grp */

	if(IsMember(gid) != SUCCESS)
	{
		ERROR("TRYING TO SEND TO GROUP NOT JOINED");
		return(_GroupNotJoined);
	}

	nbytes = 2*npoints*(sizeof(short)/sizeof(char)) ;

	message = Malloc ( nbytes*sizeof(char )) ;
	
	bcopy((char *)points, (char *)message, nbytes) ;

	LL_SendBBoardObjReq(fd, gid, lstyle, fstyle, lwidth, color, type, message, nbytes);

	Free(message) ;

	return(_NoError);

}

/*-----------------------------------------------------T_send_BBoardChar()-----*/
Status	T_send_BBoardChar(gid,letter, fstyle, ffam, fsz,color, x, y)
int		gid;		/* talk group id */
int		fstyle, ffam, fsz ;
int		color ;
char	letter ;
short	x,y ;
{
	FD fd = L_sock ;
	int		i ;

	/* 	First check for dirty business by the client */
	if(INIT_LIB_FLAG == FALSE)
		FATAL("Library structures NOT initialised");

	/* Now check whether he's actually joined the grp */
	if(IsMember(gid) != SUCCESS)
	{
		ERROR("TRYING TO SEND TO GROUP NOT JOINED");
		return(_GroupNotJoined);
	}

	LL_SendBBoardCharReq(fd, gid, fstyle, ffam, fsz, letter,color, x,y);


	return(_NoError);
}
/*---------------------------------------------------- End of send.c-----------*/
