/*----------------------------------------------------proto1.c----------------*/
/******************************************************************************/
/*                                                                            */
/*                                                                            */
/*                                Vartalaap                                   */
/*----------------------------------------------------------------------------*/
/*                     Multimedia Conferencing Software                       */
/*                                                                            */
/*                                                                            */
/*        Written by:                                                         */
/*                        Yezdezard Lashkari                                  */
/*                        Viswanath Ramachandran                              */
/*                        Sanjay Malpani                                      */
/*                     and                                                    */
/*                        Vijay Talati                                        */
/*                                                                            */
/*                                                                            */
/*                Department of Computer Science and Engineering              */
/*                Indian Institute of Technology, Bombay 400076.              */
/*                                                                            */
/******************************************************************************/


/*
 *	Routines to handle the protocol packets recd and sent by local server
 */
#include	"../common/inc.h"
#include	"../common/protodefs.h"
#include	"ex_glbls.h"
#include	"ex_routines.h"
#include	"fncproto.h"
#include	"localrout.h"

int	padlength[4] = { 0, 3, 2, 1};


/*-------------------------------------------------------ProcessError()-------*/
/*
 *	ProcessError  -- Processes any T_Error packets
 */
/*----------------------------------------------------------------------------*/

Status	ProcessError(terr)
T_Error	*terr; 
{
	ENTER_PROC("ProcessError");

	/* Now print the error fields */
	
	printf("ERROR CODE:%d\n",terr->opcode);

	return(SUCCESS);
}


/*-----------------------------------------------ProcessReqFromClient()-------*/
/*
 *	ProcessReqFromClient -- Reads a request from a given client, processes
 *	it and passes it on to global server
 */
/*----------------------------------------------------------------------------*/

int	ProcessReqFromClient(client)
FD	client;
{ 
T_Req	tReq;
char	*buf;
char	*pBuf;
Status	st;
char	*data;
int		id ;
int		port ;
int		n_fds ;
FD		*fd_arr ;


	ENTER_PROC("ProcessReqFromClient");

	/* Trying to read generic */
	if(!ReadBuffer(client, (char *)&tReq, sizeof(T_Req)))
	{
		ERROR("Couldnt read request from client");
		CloseDownClient(client);
		return (FAILURE) ; /*never reached */
	}

	 DEBUG_LEVEL1((stderr,"LOCAL SERV: Read Generic Successfully"));

	/* else */

	switch(tReq.reqType)
	{
		case	_MakeGroupReq:   
					DEBUG_LEVEL1((stderr,"MakeGroupReq recd"));

					st = ProcessMakeGroupReq( client) ;
					SendMakeGroupRep( client, st) ;

					break;

		case	_HistoryReq:   
					DEBUG_LEVEL1((stderr,"HistoryReq recd"));

					ProcessHistoryReq ( &tReq, client) ;
					break;


		case	_UserListReq:   
					DEBUG_LEVEL1((stderr,"UserListReq recd"));

					ProcessUserListReq( &tReq, client) ;
					break;

		case	_AnnotateReq:   
					DEBUG_LEVEL1((stderr,"AnnotateReq") );
					
					ProcessAnnotateReq( &tReq, client) ;
					break;

		case	_ImagemarkReq:   
					DEBUG_LEVEL1((stderr,"ImagemarkReq recd"));

					ProcessImagemarkReq( &tReq, client) ;
					break;

		case	_JoinGroupReq:   
					DEBUG_LEVEL1((stderr,"JoinGroupReq recd"));

					st =ProcessJoinGroupReq( &tReq, client, &id) ;
					SendJoinGroupRep( client, st, id) ;
					break ;


		case	_LeaveGroupReq:   
					DEBUG_LEVEL1((stderr,"LeaveGroupReq recd"));
					st = ProcessLeaveGroupReq( &tReq, client, &n_fds, &fd_arr, &id) ;
					SendLeaveGroupRep( client, id, n_fds, fd_arr) ;
				
					break ;


		case	_SendDataReq:   
					ProcessSendDataReq( &tReq, client) ;
					break ;

		case	_BBoardObjReq:
					ProcessSendBBObjReq( &tReq, client) ;
					break ;

		case	_BBoardCharReq:
					ProcessSendBBCharReq( &tReq, client) ;
					break ;


		case	_ImageStartReq:
					DEBUG_LEVEL1((stderr,"ImageStartReq received\n") );

					st = ProcessImageStartReq( &tReq, &port, &id) ;
					if ( st == FAILURE)
						SendImageStartRep( client, id, port, FALSE) ;
					else
					{
						/*Ask other clients to connect to image channel*/
						SendImageStartCmd( client, id, port) ;
						
						/* Send ack to requseter after all paths have been
							established */
						SendImageStartRep( client, id, port, TRUE) ;
					}
					break ;

		case	_AudioStartReq :
					DEBUG_LEVEL1((stderr,"AudioStartReq received \n") );

					st = ProcessAudioStartReq( &tReq, &port, &id) ;
					if (st == FAILURE) 
						SendAudioStartRep ( client, id, port, FALSE) ;
					else
					{
						/* Ask other clients to connect to the audio_channel*/
						/* And wait for connection */
						SendAudioStartCmd(client,id, port) ;
						
						/* Send ack to the requester after all paths have been
						   established */
						/* Wait for connection and add fd to ReadDescriptors */
						SendAudioStartRep( client, id, port, TRUE) ;
					}

					break ;


		case	_AudioStopReq :
				
					/* Close all the audio fds and mark the audio_channel
						of given group as free */
					ProcessAudioStopReq( &tReq, client) ;

					break ;

		case 	_AudioDataReq :

					DEBUG_LEVEL1((stderr,"AudioDataReq received") );

					ProcessAudioDataReq (&tReq, client) ;
					break ;

		case	_ImageDataReq :
					DEBUG_LEVEL1((stderr,"ImageDataReq received") );
	
					ProcessImageDataReq( &tReq, client) ;
					break ;

		default:
					ERROR("Unknown Request opcode");
					CloseDownClient(client);
					break;

	}/* End of switch */

}

/*------------------------------------------------------End of proto1.c-------*/
