/*-----------------------------------------------------sendrep.c--------------*/
/******************************************************************************/
/*                                                                            */
/*                                                                            */
/*                                Vartalaap                                   */
/*----------------------------------------------------------------------------*/
/*                     Multimedia Conferencing Software                       */
/*                                                                            */
/*                                                                            */
/*        Written by:                                                         */
/*                        Yezdezard Lashkari                                  */
/*                        Viswanath Ramachandran                              */
/*                        Sanjay Malpani                                      */
/*                     and                                                    */
/*                        Vijay Talati                                        */
/*                                                                            */
/*                                                                            */
/*                Department of Computer Science and Engineering              */
/*                Indian Institute of Technology, Bombay 400076.              */
/*                                                                            */
/******************************************************************************/


#include	<signal.h>
#include	"../common/inc.h"
#include	"../common/protodefs.h"
#include	"ex_glbls.h"
#include	"ex_routines.h"
#include	"fncproto.h"
#include	"localrout.h"

#include	<sys/stat.h>
/*--------------------------------------------------SendMakeGroupRep()--------*/
/* Send the reply to the requesting client.
 * Send id if the group is made or else send
 * error code.
/*----------------------------------------------------------------------------*/
void	SendMakeGroupRep( client, id)
FD	client	;
int	id ;
{

	MakeGroupReply	mrep ;
	char	*buf ;

	mrep.type = _T_Reply ;
	mrep.opcode	= _MakeGroupReply ;

	if ( id >= 0)
		mrep.success = TRUE ;
	else
		mrep.success = FALSE ;

	mrep.gid = id ;
	mrep.userid = client ; /* As client fds are unique, they are used as
								user ids */

	mrep.MsgNum = CurTxtPos[id][client] = 0 ;
	mrep.ImgIndx = ImageIndex[id] = 0 ;

	buf = (char *) Malloc (sizeof(MakeGroupReply)) ;

	bcopy((char *)&mrep, (char *)buf, sizeof(MakeGroupReply)) ;

	WriteToClient((FD)client, sizeof(MakeGroupReply), (char *)buf) ;

	Free(buf) ;
}


/*--------------------------------------------------SendJoinGroupRep()--------*/
/* Send success/failure indication to
 * join group request
 */
/*----------------------------------------------------------------------------*/
void	SendJoinGroupRep( fd, st, id)
FD		fd ;
Status	st ;
int		id ;
{
	JoinGroupReply	jrep ;
	T_Data	jind ;
	char			*buf ;
	FD		*fd_arr ;
	int		n_fds ,i,k;
	T_Data	td ;
	int		ln ;
	char	*pBuf;
	GRP_TAB	*ptr; 
	PARTICIPANTS_INFO	*pinfo ;
	char	*uname ;
	char	*photodata ;
	FILE	*fp ;
	struct	stat	fbuf ;
	char	*pptr ;
	char	c; 
	int		fsize; 

	jrep.type = _T_Reply ;
	jrep.opcode	= _JoinGroupReply ;

	jrep.errCode	= st ;
	if (st == _NoError)
		jrep.success = TRUE ;
	else
		jrep.success = FALSE ;

	jrep.MsgNum = htons(CurTxtPos[id][fd]) ;
	jrep.ImgIndx = htons(ImageIndex[id]) ;
	jrep.userid = fd ;

	if (DistribSocks((int)id, &fd_arr, &n_fds, fd) != SUCCESS)
	{
		ERROR("DISTRIBSOCKS FAILED") ;
		CloseDownClient(fd) ;
		return ;
	}
	jrep.no_of_peers = n_fds ;
	
	buf = (char *) Malloc (sizeof(JoinGroupReply)) ;
	bcopy((char *)&jrep, (char *)buf, sizeof(JoinGroupReply)) ;
	WriteToClient((FD)fd, sizeof(JoinGroupReply), (char *)buf) ;

	Free(buf) ;

	if ( st != _NoError)
		return ;

	ptr = grouptable[id] ;

	/* Now send the info of each user to the recently joined user */
	for (k=0; k< n_fds; k++)
	{
		td.nBytesUser = htons(strlen(ptr->pinfo->Name)) ;
		/*get the photo file size*/
		if( (fp = fopen (ptr->pinfo->pfname, "r")) == NULL)
		{
			fprintf(stderr,"Cannot open the photo file \n");
		/*WORKHERE*/	
			exit(1) ;
		} 
		fstat(fileno(fp), &fbuf) ;
		fsize = fbuf.st_size ;

		td.nBytesData = htonl(fsize );
		td.gid = id ; 
		td.userid = fd_arr[k] ;
		td.type = _T_Data ;
		td.MsgNum = htons(CurTxtPos[i][fd_arr[k]]) ;
		ln =sizeof(T_Data) + (( fsize+3) & ~3) +
						((ntohs(td.nBytesUser) +3) & ~3) ;
		buf = (char *)Malloc(ln) ;
		pBuf = buf ;
		bcopy((char *)&td, (char *)buf, sizeof(T_Data)) ;
		pBuf += sizeof(T_Data) ;

/*Send the photo */
		photodata = (char *) Malloc((fsize+2) * sizeof(char)) ;
		/*Read file data */
		pptr = photodata ;
		for (i=0; i<fsize; i++)
			*pptr++ = fgetc(fp) ;
		*pptr='\0' ;
		/* Now make the packet containing photodata*/
		bcopy((char *)(photodata), (char *)pBuf, fsize) ;
		pBuf += fsize ;
		i = padlength[fsize& 3] ;
		while( --i >= 0)
			*pBuf++ = 0 ;

		/*Username */
		bcopy((char *)(ptr->pinfo->Name), (char *)pBuf, ntohs(td.nBytesUser)) ;
		pBuf += htons(td.nBytesUser );
		i = padlength[htons(td.nBytesUser) & 3] ;
		while( --i >= 0)
			*pBuf++ = 0 ;
		

		/* Send the data to client */

		WriteToClient((FD)fd, ln, (char *)buf) ;


		ptr = ptr->nxt ;

		Free(buf) ;
		Free(photodata) ;
		fclose(fp) ;
	}
	

	/* Also send to all other participants of the group an indication that
		this user has joined the group */
	if ( st == _NoError)
	{
		jind.type = _NU_Data ; /* New User data*/
		jind.gid = (unsigned short)id;
		jind.userid = (unsigned short)fd ;
		UserName( fd, &uname) ;
		jind.nBytesUser = htons(strlen(uname)) ;
/*Send the photo here */
		pinfo = getpinfo(fd) ;
		/*Open the phot file */
		fp = fopen (pinfo->pfname, "r") ;
		fstat(fileno(fp), &fbuf) ;
		fsize = fbuf.st_size ;
		jind.nBytesData = htonl(fsize); 
		ln =sizeof(T_Data) + (( fsize+3) & ~3) +
						((strlen(uname) +3) & ~3) ;
		buf = (char *)Malloc(ln) ;
		pBuf = buf ;
		bcopy((char *)&jind, (char *)buf, sizeof(T_Data)) ;
		pBuf += sizeof(T_Data) ;

/*Send the photo */
		photodata = (char *) Malloc(fsize * sizeof(char)) ;
		/*Read file data */
		pptr = photodata ;
		for (i=0; i<fsize; i++)
			*pptr++ = fgetc(fp) ;
		*pptr='\0' ;

		bcopy((char *)(photodata), (char *)pBuf, fsize) ;
		pBuf += fsize ;
		i = padlength[fsize & 3] ;
		while( --i >= 0)
			*pBuf++ = 0 ;

		/*Username */
		bcopy((char *)(uname), (char *)pBuf, strlen(uname)) ;
		pBuf += strlen(uname) ;
		i = padlength[strlen(uname) & 3] ;
		while( --i >= 0)
			*pBuf++ = 0 ;

		for (i=0; i<n_fds; i++)
		{
			if ( (FD)fd_arr[i] == fd)
				continue ;
			WriteToClient( fd_arr[i], ln, (char *)buf) ;
		}

		Free(buf) ;
		Free(photodata) ;
		fclose(fp) ;
	}	/*if*/
	
}

/*-------------------------------------------------SendLeaveGroupRep()--------*/
/* Send leave group request reply
 */
/*----------------------------------------------------------------------------*/
SendLeaveGroupRep ( client, id, n_fds, fd_arr)
FD		client;
int		id ;
int		n_fds ;
FD		*fd_arr ;
{
	
	LeaveGroupReply	lrep ;
	T_Data			lind ;
	int		i;
	char	*buf ;

	ENTER_PROC("SendLeaveGroupReq") ;
	lrep.type = _T_Reply ;
	lrep.opcode = _LeaveGroupReply ;

	lrep.success = TRUE ;

	buf = (char *) Malloc (sizeof(LeaveGroupReply)) ;
    bcopy((char *)&lrep, (char *)buf, sizeof(LeaveGroupReply)) ;
    WriteToClient((FD)client, sizeof(LeaveGroupReply), (char *)buf) ;
 
	Free(buf) ;

	/* Write to all other participants indicating that this user is leaving
	   the group.*/

	lind.type = _LU_Data ;
	lind.gid = (unsigned short)id ;	
	lind.userid = (unsigned short)client ;

	buf = (char *)Malloc(sizeof(T_Data)) ;
	bcopy((char *)&lind, (char *)buf, sizeof(T_Data)) ;
/****
	if (DistribSocks((int)id, &fd_arr, &n_fds, client) != SUCCESS)
	{
		ERROR("DISTRIBSOCKS FAILED") ;
		CloseDownClient(client) ;
		return ;
	}
*****/
	for (i=0; i<n_fds; i++)
	{
		if ( (FD)fd_arr[i] == client)
			continue ;
		WriteToClient( fd_arr[i], sizeof(T_Data), (char *)buf) ;
	}

	Free(buf) ;
}
	

/*--------------------------------------------------SendUserListData()--------*/
/* Send the data of all participants (stored in participants info table)
 * to the client who requested it.
 */
/*----------------------------------------------------------------------------*/
void	SendUserListData ( client, list)
FD		client ;
char	*list ;
{

	T_Data	tdata ;
	int		ln ;
	char	*buf, *pBuf ;
	int		i ;

	/* Fill in the T_Data Struct */

	tdata.type = _UL_Data ;
	tdata.nBytesData = strlen(list) ;
	tdata.nBytesUser = 0 ;

	/* Make the packet to send */
	
	ln = sizeof (T_Data) + ((tdata.nBytesData +3) & ~3) +
						((tdata.nBytesUser + 3) & ~3 ) ;

	buf = (char *) Malloc( ln) ;
	pBuf = buf ;

	bcopy((char *)&tdata, (char *)buf, sizeof(T_Data)) ;

	pBuf += sizeof(T_Data) ;

	bcopy((char *)list, (char *)pBuf, tdata.nBytesData) ;

	pBuf += tdata.nBytesData ;

	i = padlength[tdata.nBytesData & 3] ;

	while(--i >= 0)
		*pBuf++ = 0 ;

	WriteToClient( client, ln, (char *)buf) ;

	Free(buf) ;
}

/*-------------------------------------------------SendAudioStartRep()--------*/
/* Send ack. and prot number or error report to
 * audio channel requesting client
 */
/*----------------------------------------------------------------------------*/
void	SendAudioStartRep( client, id, port, status) 
FD		client ;
int		id ;
int		port ;
int	status ;
{

	AudioStartReply		asrep ;
	char				*buf ;
	struct sockaddr_in	cli_addr ;
	int					clilen ;
	FD					fd ;

	ENTER_PROC("SendAudioStartRep") ;

	/* Make the pakcet to send */

	asrep.success = status ;
	asrep.type = _T_Reply ;
	asrep.opcode	= _AudioStartReply ;
	asrep.port = port ;
	asrep.gid = id ;

	buf = (char *)Malloc(sizeof(AudioStartReply)) ;

	bcopy((char *)&asrep, (char *)buf, sizeof(AudioStartReply) ) ;

	WriteToClient( client, sizeof(AudioStartReply) , (char *) buf) ;

	Free(buf) ;

	if (status == FALSE) /* Sent a negative ack:so dont wait for connection*/
		return ;

	/* Wait for connection on audio port */
	fd = accept(ausockfd, NULL, 0) ;

	if ( fd < 0 )
	{
		perror("Server: accept error on audio socket " );
		/* Do error processing here *//*WORKHERE*/ 
		CloseDownClient(client) ;
		return ;
	}


	/* Add the new fd to ReadDescriptors to listen at its data */
	AddFD(fd) ;

	/* Add in groupaufds */
	/*groupaufds[id][participantsnumber[id]-1] = fd ;*/ /*WORKHERE*/ 

	return ;
}

/*-------------------------------------------------SendImageStartRep()--------*/
/* Send ack. and prot number or error report to
 * image channel requesting client
 */
/*----------------------------------------------------------------------------*/
void	SendImageStartRep( client, id, port, status) 
FD		client ;
int		id ;
int		port ;
int	status ;
{

	ImageStartReply		isrep ;
	char				*buf ;
	struct sockaddr_in	cli_addr ;
	int					clilen ;
	FD					fd ;

	ENTER_PROC("SendImageStartRep") ;

	/* Make the pakcet to send */

	isrep.success = status ;
	isrep.type = _T_Reply ;
	isrep.opcode	= _ImageStartReply ;
	isrep.port = port ;
	isrep.gid = id ;

	buf = (char *)Malloc(sizeof(ImageStartReply)) ;

	bcopy((char *)&isrep, (char *)buf, sizeof(ImageStartReply) ) ;

	WriteToClient( client, sizeof(ImageStartReply) , (char *) buf) ;

	Free(buf) ;

	if (status == FALSE) /* Sent a negative ack:so dont wait for connection*/
		return ;

	/* Wait for connection on image port */
	fd = accept(imgsockfd, NULL, 0) ;

	if ( fd < 0 )
	{
		perror("Server: accept error on audio socket " );
		/* Do error processing here *//*WORKHERE*/ 
		CloseDownClient(client) ;
		return ;
	}

	/* Add the new fd to ReadDescriptors to listen at its data */
	AddFD(fd) ;

	return ;
}

	
/*-----------------------------------------------End of sendrep.c-------------*/

