/*------------------------------------------------------startup.c-------------*/
/******************************************************************************/
/*                                                                            */
/*                                                                            */
/*                                Vartalaap                                   */
/*----------------------------------------------------------------------------*/
/*                     Multimedia Conferencing Software                       */
/*                                                                            */
/*                                                                            */
/*        Written by:                                                         */
/*                        Yezdezard Lashkari                                  */
/*                        Viswanath Ramachandran                              */
/*                        Sanjay Malpani                                      */
/*                     and                                                    */
/*                        Vijay Talati                                        */
/*                                                                            */
/*                                                                            */
/*                Department of Computer Science and Engineering              */
/*                Indian Institute of Technology, Bombay 400076.              */
/*                                                                            */
/******************************************************************************/

/*
 *	This file contains routines to startup the local server,
 *	Connect to the global server, accept connections from
 *	the client library, and initialise local server data
 *	structures
 */

#include	"../common/inc.h"
#include	"../common/protodefs.h"		/* For the protocol defs */
#include	"ex_glbls.h"
#include	"ex_routines.h"
#include	"fncproto.h"
#include	"localrout.h"


static	int	ON = 1;		/* Used in ioctl calls */

static	int	padlength[4] = {0, 3, 2, 1};

/*---------------------------------------------CreateWellKnownPort()----------*/
/* Create a well known port at which the server listens,
 * The port is BASE(5000) + offset( form command line args)
 */
/*----------------------------------------------------------------------------*/
FD	CreateWellKnownPort(wellknownport)
int	wellknownport;		
{
	FD	sock;


	struct	sockaddr_in	server;	/* Local server on INET CONNECTION*/
	int	length = sizeof(server) ;
	ENTER_PROC("CreateWellKnownPort");


	if((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0)
    {
                perror("Couldnt open INET stream socket");
                FATAL("Couldnt create well known port");
    }
 
	(void)setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)NULL, 0);
#ifdef	SUN
    (void)setsockopt(sock, SOL_SOCKET, SO_USELOOPBACK, (char *)NULL, 0);
#endif


	bzero((char *)&server, sizeof(server)) ;	 

	server.sin_family = AF_INET;
	server.sin_addr.s_addr = htonl(INADDR_ANY);
	server.sin_port =htons(wellknownport);
 
 
	if( bind(sock, (struct sockaddr *)&server, sizeof(struct sockaddr_in)))
	{
                perror("Couldnt bind name to well known INET socket");
                exit(1);
	}
 
 
    if(getsockname(sock, &server, &length))
        SYSFATAL("getsockname failed for well known port");
    DEBUG_LEVEL1((stderr,"Well known socket port:%d\n",ntohs(server.sin_port)));
 

	/* now set it ready to listen */

	if(listen(sock, BACKLOG) < 0)
	{
		perror("listen failed on well known socket\n");
		FATAL("Listen failed");
	}

	AddFD(sock);		/* Add to mask to select from */

	return(sock);

}

/*-------------------------------------------------------ValidUser()----------*/
/* Check the validity of the connecting client
 * Right now, dummy. Allows all users
 */
/*----------------------------------------------------------------------------*/
int	ValidUser(authproto, authstr)
char	*authproto;
char	*authstr;
{
	ENTER_PROC("ValidUser");
	return(TRUE);
}

/*---------------------------------------------------SendSetupInfo()----------*/
/*
 *	SendSetupInfo -- If valid user then send SUCCESS else FAILURE
 *	If failure then send reason.
 */
/*----------------------------------------------------------------------------*/
void	SendSetupInfo( conn, valid)
FD	conn;
Bool	valid;
{
	char	*setupinfo;
	char	*pBuf;
	int	lenblock;
	int	i;
	int	lr = 0;

	ConnSetupPrefix ccp ; 

	ENTER_PROC("SendSetupInfo");

	lenblock = sizeof(ConnSetupPrefix);

	if(valid == FALSE)
	{
		ccp.success = FALSE;
#define STR "Authorisation Invalid : Yezdi doesnt like you"
		lr = ccp.lengthReason = strlen(STR);
		lenblock += ((lr + 3) & ~3);

	}
	else
	{
		ccp.success = TRUE;
	 	ccp.lengthReason = 0;

	}
	
	setupinfo = (char *)Malloc(lenblock);
	pBuf = setupinfo ;

	bcopy((char *)&ccp, setupinfo, sizeof(ConnSetupPrefix));
	pBuf += sizeof(ConnSetupPrefix);

	if(valid == FALSE)
	{
		bcopy(STR, pBuf, (lr * sizeof(char)));
		pBuf += lr;

		/* pad to a multiple of 4 bytes */
		i = padlength[lr & 3];
		while(--i >= 0)
			*pBuf++ = 0;
#undef	STR
	}
	/* else */
	/* Do nothing */

	/* now send this back to client */

	WriteToClient(conn, lenblock, (char *)setupinfo);
	return;
}

/*---------------------------------------------AcceptClientConnection()-------*/
/*
 *	This routine is called iff select indicates that there is a client
 *	waiting to connect up on the well known port. Else accept will
 *	simply block --- something we can't afford as we have to service
 *	other requests
 */
/*----------------------------------------------------------------------------*/

FD	AcceptClientConnection(wellknownsocket, usrname, photodataptr, psizeptr, pid)
FD	wellknownsocket;
char	*usrname;		/*  Return */
char	**photodataptr ;
int		*psizeptr ;
int	*pid;		/*  Return */
{
	register FD client;
	char	authproto[100]; /*WORKHERE*/
	char	authstring[100];
	char 	*username;		
	int	slen;
	register LL_ConnClientPrefix	*llccp
		= (LL_ConnClientPrefix *)Malloc(sizeof(LL_ConnClientPrefix));
	
	ENTER_PROC("AcceptClientConnection");

	if((client = accept(wellknownsocket,NULL,0)) < 0)
	{
		perror("accept failed\n");
		ERROR("Couldnt connect to client");
		return(-1);
	}

	/* a few parameter settings */
 
    (void)setsockopt(client, SOL_SOCKET, SO_REUSEADDR, (char *)NULL, 0);
#ifdef SUN
    (void)setsockopt(client, SOL_SOCKET, SO_USELOOPBACK, (char *)NULL, 0);
#endif

/*Commented: Vijay 11/12/92 */
	/*ioctl(client, FIONBIO,&ON);	*//* Set non blocking  */

	/* Now check authorisation info */

	username = (char *)Malloc(MAXNAMESZ * sizeof(char));

	if(!ReadBuffer( client, (char *)llccp, sizeof(LL_ConnClientPrefix)))
	{
		ERROR("Couldnt Read Client Prefix");
		return(-1);
	}
	/* else */
	/* Validate he's the user */

	slen = (ntohs(llccp->nBytesAuthProto) + 3) & ~3;
	if(slen)
	{
		if(!ReadBuffer(client, authproto, slen))
		{
			ERROR("Length error reading authorisation info");
			close(client);
			return(-1);
		}
		authproto[slen] = '\0';
		DEBUG_LEVEL3((stderr,"Auth proto string:%s\n",authproto));
	 }

	slen = (ntohs(llccp->nBytesAuthString) + 3) & ~3;
	if(slen)
	{
		if(!ReadBuffer(client, authstring, slen))
		{
			ERROR("Length error reading authorisation info");
			close(client);
			return(-1);
		}
		authstring[slen] = '\0';
		DEBUG_LEVEL3((stderr,"Auth string:%s\n",authstring));
	}

	*pid = ntohl((long)llccp->pid);

	DEBUG_LEVEL3((stderr,"pid retd:%d\n", *pid));

	slen = (ntohs(llccp->nBytesUserName) + 3) & ~3;
	if(slen)
	{
		if(!ReadBuffer(client, username, slen))
		{
			ERROR("Length error reading user name");
			close(client);
			return(-1);
		}
		DEBUG_LEVEL1((stderr,"username:%s\n",username));
		username[slen] = '\0';

		strcpy(usrname, username);
		usrname[slen] = '\0';
	 }

	
	slen = (ntohl(llccp->nBytesUserPhoto) + 3) & ~3;
	(*psizeptr) = ntohl(llccp->nBytesUserPhoto );
	(*photodataptr) = (char *)Malloc(slen+1) ;
	if(slen)
	{
		if(!ReadBuffer(client, *photodataptr, slen))
		{
			ERROR("Length error reading user name");
			close(client);
			return(-1);
		}
		(*photodataptr)[slen] = '\0';

	 }

	/* Now Validate */

	if( ValidUser(authproto, authstring) != TRUE )
	{
		ERROR("User not valid ");
		SendSetupInfo(client, FALSE);
		close(client);		/* Actually send him info later */
		return(-1);
	}
	/* else */
	SendSetupInfo(client, TRUE);

	/* If we've reached here then it's VOKAY TYPES */

	return(client);

}
/*--------------------------------------------------End of startup.c----------*/
